/***************************************************************************
                          aspscanlist.cpp  -  description
                             -------------------
    begin                : sam oct 5 2002
    copyright            : (C) 2002 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "aspscanlist.h"
#include "astroprocess.h"
#include "astrodefs.h"
#include "astroresources.h"
#include "stopscan.h"

AspScanList::AspScanList(ChartBaseParams* cbp, ObjsVector* ov) : GraphicListBase(cbp, ov)
{
  ct = AspScList;
  cm = GraphicMode;
  DualChart = TRUE;
  Sci =  new ScanClient(this, ++(Asr->ScanId));
}

AspScanList::~AspScanList()
{
	~Dct;
	delete Sci;
}

bool AspScanList::PostRecompute()
{
	if(Running) return FALSE;
	End();
	Running = TRUE;
	int s = Search(2, Any, TRUE), d, m, y, h, min;
	Asp = Asr->Asp;
  Second = (s != -1) ? (*this)[s] : 0;
  if(!Second) return FALSE;
  AstroDate orig(*Second);
  AstroString as;
  Second->GetDate(&d, &m, &y, &h, &min);
  h = min = 0;
  switch(Asp.Ss)
  {
		case Current_Day   : break;
		case Current_Month : d = 1; break;
		case Current_Year  : 
		case Years         : d = 1; m = 1;
	};
	Second->SetDate(d, m, y, 0, 0);
	d = m = y = h = min = 0;
	switch(Asp.Ss)
  {
		case Current_Day   : h = 23; min = 59; break;
		case Current_Month : m = 1; break;
		case Current_Year  : y = 1; break;
		case Years         : y = Asp.NbYears;
	}
	Dc = Second->GetDataComp();
	Dc.julday = IncTime(Second->GetJd(), min, h, d, m, y);
	Dct = Dc;
	AdList.Clear();
	QString ScanIntfName = Sci->objId();
	Ss = new StopScan(this);
	Second->GetInterface()->ScanAsp(Dct, Second->GetJd(), ScanIntfName, Asp.ToInt());
	while(!Sci->IfEnd())
		P;
	*SC(AstroDate*, Second) = orig;
	delete Ss;
	Running = FALSE;
	Begin();
	return TRUE;
}

bool AspScanList::Redisplay()
{
	struct AspData* ad;
	AstroDate dat(*Second);
	AstroString as, as1;
	Init();
  int l = W / 5, l2 = l / 4;
	const int tab[] = { 0, l + l2, 2 * l + l2, 3 * l + l2, 4 * l + l2, 5 * l + l2 };
	SetTabList(tab);
	SetTextStyle(Title);
	DrawText(i18n("Aspects Scan List for"));
	NewLine();
	GetTitle(as);
	DrawText(as);
	NewLine();
	SetTextStyle(TabText);
	DrawTabText("Date");
	DrawTabText(i18n("Object 1"));
	DrawTabText(i18n("Aspect"));
	DrawTabText(i18n("Object 2"));
	DrawTabText(i18n("Comment"));
	NewLine();
	NewLine();
	for(ad = AdList.First(); ad; ad = AdList.Next())
	{
		switch(ad->ak)
		{
		  case ExAsp:
				dat.SetJd(ad->jd);
				dat.GetDate(as);
				as += "   ";
				as += dat.GetTime(as1);
				DrawTabText(as);
				as = ObjName(ad->a);
				as += ad->apos < 0.0 ? "[" : "(";
				as += (*Asg) < Asg->GetSign(fabs(ad->apos));
				as += ad->apos < 0.0 ? "]" : ")";
				DrawTabText(as);
				DrawTabText(*Asr->AspectNames[ad->asp]);
				as = ObjName(ad->b);
				as += ad->bpos < 0.0 ? "[" : "(";
				as += (*Asg) < Asg->GetSign(fabs(ad->bpos));
				as += ad->bpos < 0.0 ? "]" : ")";
				DrawTabText(as);
				if(ad->a == ad->b && ad->asp == Conjunction)
					DrawTabText(Sprintf("%a return", &as, &ObjName(ad->a)));
				break;
		 case ChangeDir:
			  dat.SetJd(ad->jd);
			  dat.GetDate(as);
				as += "   ";
				as += dat.GetTime(as1);
				DrawTabText(as);
				as = ObjName(ad->a);
				as += " ";
				as += (*Asg) < Asg->GetSign(fabs(ad->apos));
				DrawTabText(as);
				DrawTabText(ad->apos < 0.0 ? i18n("Direct->Retro") : i18n("Retro->Direct"));
				break;
		};
		NewLine();
	}
	return TRUE;
}

const AstroString& AspScanList::ObjName(int i) const
{
	if(i < 0 || i > Last_Star)
    return *Asr->ObjError;
  if(i > Last_Planet && i < First_Exotic)
    return *Asr->ObjError;
  if(i > Last_Exotic && i < Ascendant)
    return *Asr->ObjError;
  if(i >= First_Star && i <= Last_Star)
    return *Asr->StarsNames[i - First_Star];
  if(i >= First_Exotic && i <= Last_Exotic)
    i = i - First_Exotic + 23;
  if(i >= Ascendant && i <= House_12)
    i = i - Ascendant + 31;
  return *Asr->ObjNames[i];
}

void AspScanList::DrawDate(int i, double ipos, int j, double jpos, int asp, double julday)
{
	struct AspData* ad = new struct AspData;
	ad->ak = ExAsp;
	ad->a = (enum Objs)i;
	ad->apos = ipos;
	ad->b = (enum Objs)j;
	ad->bpos = jpos;
	ad->asp = (enum Aspects)asp;
	ad->jd = julday;
	AdList.Append(ad);
}

void AspScanList::DrawChange(int i, double ipos, double julday)
{
	struct AspData* ad = new struct AspData;
	ad->ak = ChangeDir;
	ad->apos = ipos;
	ad->a = (enum Objs)i;
	ad->jd = julday;
	AdList.Append(ad);
}
