/***************************************************************************
                          astroaspects.cpp  -  description
                             -------------------
    begin                : Mon Apr 30 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astroaspects.h"
#include "astroresources.h"
#include "astroobjs.h"

_AstroAspects::_AstroAspects(_AstroRestrictions* ar)
{
  Asprst = ar->Asprst;
  Restr   = ar->Restr;
  IsMinor = ar->IfMinor();
}

AstroAspects::AstroAspects(AstroRestrictions* ar) : _AstroAspects(SC(_AstroRestrictions*, ar))
{
  AspChars = ASPCHARS;
}

enum Aspects _AstroAspects::IsAspect(enum Objs oa, const Values& a,
	enum Objs ob, const Values& b, enum AspectAccuracy& side) const
{
  int i;
  double orb;
  double angle, aasp;
  const double angle_a = a.Get(oa), angle_b = b.Get(ob), fa = fabs(angle_a - angle_b);
  side = Exact;
  for(i = Conjunction; i <= Last_Major; i++)
  {
    orb = Asprst[i].Orb + Restr[oa].OrbAdd + Restr[ob].OrbAdd;
    if(orb < 0.0) continue;
    angle = fa;
    if(angle - orb > 180.0)
      angle = 360.0 - angle;
    aasp = (*this) * (enum Aspects)i;
    if((angle < aasp + orb) && (angle > aasp - orb) && (*this) == (enum Aspects)i)
    {
      angle = DiffAngle(angle_a, angle_b) - aasp;
      if(angle > 0.0)
      	side = Separating;
      else if(angle < 0.0)
      	side = Applying;
      return (enum Aspects)i;
     }
  }
  if(IsMinor)
  {

    for(i = First_Minor; i < Unknown_Aspect; i++)
    {
      orb = Asprst[i].Orb; // + Restr[oa].OrbAdd + Restr[ob].OrbAdd;
      if(orb < 0.0) continue;
      if(i == Parallel || i == Antiparallel)
      {
       const double pa =  a.Get(oa, 1), pb = b.Get(ob, 1), pab = pa * pb, dp = fabs(fabs(pa) - fabs(pb));
       bool parcond = (oa <= Pluto || oa == Black_Moon || (oa >= Chiron && oa <= Vesta) ||
        (oa >= First_Exotic && oa <= Last_Exotic) || oa >= First_Star) && (ob <= Pluto ||
        ob == Black_Moon || (ob >= Chiron && ob <= Vesta) || (ob >= First_Exotic &&
        ob <= Last_Exotic) || ob >= First_Star);
       if(i == Parallel && pab >= 0.0 && parcond && dp <= orb) return (enum Aspects)i;
       if(i == Antiparallel && pab < 0.0 && parcond && dp <= orb) return (enum Aspects)i;
       continue;
      }
      angle = fa;
      if(angle - orb > 180.0)
      angle = 360.0 - angle;
      aasp = (*this) * (enum Aspects)i;
      if((angle < aasp + orb) && (angle > aasp - orb) && (*this) == (enum Aspects)i)
      {
        angle = DiffAngle(angle_a, angle_b) - aasp;
        if(angle > 0.0)
         side = Separating;
        else if(angle < 0.0)
         side = Applying;
        return (enum Aspects)i;
       }
    }
  }
  return Unknown_Aspect;
}

char AstroAspects::GetAspChar(I) const
{
	return AspChars[i];
}

enum Aspects AstroAspects::IsAspect(enum Objs oa, const AstroObjs& a,
  	enum Objs ob, const AstroObjs& b, enum AspectAccuracy& side) const
{
	return _AstroAspects::IsAspect(oa, a.Vals, ob, b.Vals, side);
}

void AstroFoundAspectsTable::GetTable(const _AstroAspects& aa, const Values& a, const Values& b, bool ifsameplanet,
		const _AstroRestrictions& ar, const _AstroRestrictions& br)
{
  Max_x = Last_Planet;
  Max_y = Last_Planet;
  if(ar.IfExotic())
    Max_x = Last_Exotic;
  if(br.IfExotic())
    Max_y = Last_Exotic;
  if(ar.IfHouses())
    Max_x = House_12;
  if(br.IfHouses())
    Max_y = House_12;
  if(ar.IfStars())
    Max_x = Last_Star;
  if(br.IfStars())
    Max_y = Last_Star;
  IfSamePlanet = ifsameplanet;
  Reset(Max_x, Max_y);
  TestAspects(aa, a, b, ar, br);
}

void AstroFoundAspectsList::GetList(const AstroAspects& aa, const AstroObjs& a, const AstroObjs& b, bool ifsameplanet)
{
	Clear();
	AstroFoundAspectsTable::GetTable(aa, a.Vals, b.Vals, ifsameplanet, *(~a), *(~b)); 
}

void AstroFoundAspectsTable::Reset(int a, int b)
{
	a++;
	b++;
	if(Already) delete Already;
	Already = new asp[a * b];
	memset(Already, Unknown_Aspect, sizeof(asp) * a * b);
}

static int Count(const AstroObjs& a)
{
	int count = 0, j;
	for(j = Sun; j <= Last_Planet; j++)
		if(a == j)
			count++;
	if(a.IfHouses())
		for(j = Ascendant; j <= House_12; j++)
			if(a == j)
				count++;
	return count;
}

void AstroFoundAspectsList::ComputeMidP(const AstroObjs& a, const AstroObjs& b, bool ifsameplanet)
{
	int i, k = 0;
	IfSamePlanet = ifsameplanet;
	if(MidP)
		delete MidP;
	NbMP = ::Count(a) * (ifsameplanet ? ::Count(b) : ::Count(b) -1);
	MidP = new struct MidPoint[NbMP];
	for(i = Sun; i <= Last_Planet; i++)
		if(a == i)
			GetMidP(a * i, b, i, &k);
	if(a.IfHouses())
		for(i = Ascendant; i <= House_12; i++)
			if(a == i)
				GetMidP(a * i, b, i, &k);
}

void AstroFoundAspectsList::GetMidP(double d, const AstroObjs& b, int i, int* k)
{
	int j;
	for(j = Sun; j <= Last_Planet; j++)
	{
		if(!IfSamePlanet && i == j) continue;
		if(b == j)
		{
			MidP[*k].Mp = MidPoint(d, b * j);
			MidP[*k].Oa = (enum Objs)i;
			MidP[*k].Ob = (enum Objs)j;
			(*k)++;
		}
	}
	if(b.IfHouses())
		for(j = Ascendant; j <= House_12; j++)
		{
			if(!IfSamePlanet && i == j) continue;
			if(b == j)
			{
				MidP[*k].Mp = MidPoint(d, b * j);
				MidP[*k].Oa = (enum Objs)i;
				MidP[*k].Ob = (enum Objs)j;
				(*k)++;
			}
		}
}

void AstroFoundAspectsTable::TestAspects(const _AstroAspects& aa, const Values& a, const Values& b,
	const _AstroRestrictions& ar, const _AstroRestrictions& br)
{
  int i, j;
  bool s = ar.IfStars() && br.IfStars();
  enum AspectAccuracy side;
  for(i = Sun; i <= Max_x; i++)
  {
    for(j = Sun; j <= Max_y; j++)
    {
			if(!(ar == i) || !(br == j) || (i == j && !IfSamePlanet)) continue;
      if(AL(i, j) != Unknown_Aspect) continue;
      if(s && i >= First_Star && j >= First_Star) continue;
      if(!IfSamePlanet && i == North_Node && j == South_Node) continue;
      if(!IfSamePlanet && i >= Ascendant && i <= House_12 && j >= Ascendant && j <= House_12)
        continue;
      enum Objs oa = (enum Objs)i, ob = (enum Objs)j;
      enum Aspects asp = aa.IsAspect(oa, a, ob, b, side);
      if(asp != Unknown_Aspect)
      {
				AL(i, j) = asp;
				if(!IfSamePlanet) AL(j, i) = asp;
				PutAspect(aa, asp, oa, a, ob, b, side);
      }
   }
  }
}

void AstroFoundAspectsList::PutAspect(const _AstroAspects& aa, enum Aspects asp, enum Objs oa,
		const Values& a, enum Objs ob, const Values& b, enum AspectAccuracy side)
{
	Append(new AspectFound(aa, asp, oa, a, ob, b, side));
}

void AstroFoundAspectsList::TestAspects(const AstroAspects& aa, const AstroObjs& a, const AstroObjs& b)
{
	AstroFoundAspectsTable::TestAspects(aa, a.Vals, b.Vals, *(~a), *(~b));
}

AspectFound::AspectFound(const _AstroAspects& aa, enum Aspects asp, enum Objs oa, const Values& a,
	enum Objs ob, const Values& b, enum AspectAccuracy side)
{
  const struct Rest* Restr = aa.GetRest();
  Aa = (_AstroAspects *)&aa;
  Asp = asp;
  Oa = oa;
  Ob = ob;
  Angle_a = a.Get(oa);
  Angle_b = b.Get(ob);
  AspectSide = side;
  Orb = (*Aa) / asp + (asp <= Inconjunct) ? Restr[oa].OrbAdd + Restr[ob].OrbAdd : 0;
  DiffLong = DiffAngle(fabs(a.Get(oa, 1)), fabs(b.Get(ob, 1)));
}

enum AspectAccuracy AspectFound::GetAccuracy()
{
  double diff = fabs(Angle_a - Angle_b) - (*Aa) * Asp;
  int x = 100 * (int)(diff / Orb);
  if(x == 0)
    return Exact;
  else if(x <= 30)
    return Good;
  else if(x <= 60)
    return Medium;
  else
    return Poor;
}
