/***************************************************************************
                          astrochartbase.h  -  description
                             -------------------
    begin                : Sat Oct 14 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROCHARTBASE_H
#define ASTROCHARTBASE_H

#include "astroobjs.h"
#include <ostream.h>

#ifdef KDE
# include <qobject.h>
#endif

#define ACB (*Acb)

typedef AstroObjs* ObjsVector[NBDATA];

class AstroGraphics;
class AstroFoundAspectsList;
class QWidget;

/** This class provides additional params for the displayed charts,
    such as the as the AstroInterface pointer */

class ChartBaseParams : public ChartParams
{
 public:

  inline ChartBaseParams(ChartParams* cp, AstroInterface* aif = Asf) : ChartParams(*cp)
  {
		Qw = 0; Ag = 0; Stream = 0; Aif = aif;
	}

  AstroInterface* Aif;
  AstroGraphics*  Ag;
  Ostream*        Stream;
#ifdef KDE
  QWidget*        Qw;
#endif
};

/**This classs is the base class for all the charts
  *@author Christophe Gros
  */
class AstroChartBase : public QObject
{
 Q_OBJECT

public:
  friend class AstroChart;

  enum ChartType ct;
  enum ChartMode cm;

  enum SearchType { SingleOnly, NonSingle, Any }; 

  // if TRUE, the chart MUST have at least 2 AstroObjs available.
  bool DualChart;

  // If TRUE, the aspects are computed
  bool IfAspects;

  // TRUE when long computing
  bool Running;

  /** Constructor */
  AstroChartBase(ChartBaseParams *cbp, ObjsVector* ov);
  
  /** Destructor */
  virtual ~AstroChartBase();
		
  /** Returns the ith AstroObjs pointer */
  inline AstroObjs* operator[](int i) const { return (*Ob)[i]; }

  /** Returns the number of active AstroObjs */
  int Count() const;

  /** The abstract method that will redisplay the page 'page'
  	of the current chart */
  virtual bool Redisplay() = 0;

  /** Post recomputing routine */
  virtual inline bool PostRecompute() { return TRUE; }

  /** The virtual specific parameters setting for the chart.
 @param str Possible data. If str == 0, the Settings are interactive */
  virtual bool Settings(const char* str = 0) {return TRUE;}

  /** Gives the chart's window title */
  void GetTitle(AstroString& as) const;

  /** Gets the first available AstroObjs
   @param single if single == TRUE, only returns the first single
   @return the index of the first avail AstroObjs (0..3) or -1
  */
  int First(bool single = FALSE) const { return Search(1, single ? SingleOnly : Any, TRUE); };

  /** Says if the AstroObjs i is available */
  bool operator == (int i) const;

  /** Search for the i th ring having the conditions st and visible
      @param i the ring rank
      @param if Single only, non-single or any
      @param visible rings only
      @return the corresponding ring or -1
  */
  int Search(int r = 1, enum SearchType st = Any, bool visible = FALSE) const;

  /** Returns an aspects list, if any */
  inline virtual const AstroFoundAspectsList* GetAspectsList() const { return 0; }

	bool IfPage;

	signals:

	void NewPage();

 protected:

  bool IfRedrawChart;
  ChartBaseParams *Cbp;
  /** Executed after any SetData
      @param i the index of the added AstroData
  */ 
  virtual void PostSetData(int i);
  
 private:
  
  ObjsVector* Ob;
};
#endif
