/***************************************************************************
                          astrointerface.h  -  description
                             -------------------
    begin                : Fri May 18 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROINTERFACE_H
#define ASTROINTERFACE_H

#include "astroresources.h"

#ifdef KDE
#include "astroprocess_stub.h"
#endif

#include "astrodefs.h"
#include "astrostring.h"

class AstroRestrictions;

/** This class provides an interface to the astro calculation process */
class AstroInterface:
#ifdef KDE
 public AstroComputing_stub
#endif
{

 public:

  /** Constructor
      @param app the server name
      @param id the served class id
  */
  AstroInterface(AstroString app, AstroString id);

  /** Destructor */
  ~AstroInterface() { ~Art; }

  /** Sets timezone file */
  void SetTZ(const char* file = 0, bool set = TRUE);

  /** Gives the path for the swe positions files */
  void SetSweDataPath(const char* path);

  /** Sets the restriction group */
  void SetRestrict(const AstroRestrictions* ar, int restr = 0);

  /** Returns the error text line if any */ 
  void GetError(AstroString&);

  /** Returns zoneinfo buffer (internal use only */
  inline const char *GetZib() const { return ZoneInfoBuf; }

 private:

  char ZoneInfoBuf[BUFSIZE];
  ARTransfer Art;
};

class ScanInterface
#ifdef KDE
: public ScanClient_stub
#endif
{
	public:

	ScanInterface(AstroString app, AstroString id);

  /** Destructor */
  ~ScanInterface() {}
};

#endif


