/***************************************************************************
                          astroobjs.cpp  -  description
                             -------------------
    begin                : Sat Sep 23 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astroobjs.h"
#include "astrochart.h"
#include "astrosigns.h"
#include "astroresources.h"
#include "astrographics.h"

extern "C"
{
#include "sweph.h"
};

AstroObjs::AstroObjs(AstroInterface* aif) : AstroData(aif), ChartParams(*Asr)
{
  Ac = 0;
  Ith = 0;
  Cst = Single;
  RestNb = 0;
  PlaChars = PLACHARS;
  Show = TRUE;
  DefSweFlags = 0;
  Ar = Drs;
}

AstroObjs::AstroObjs(AstroInterface *aif, AstroChart *ac, int ith) : AstroData(aif), ChartParams(*ac->Cbp)
{
  Ac = ac;
  Ith = ith;
  Cst = Single;
  PlaChars = PLACHARS;
  Show = TRUE;
  DefSweFlags = 0;
  Ar = Drs;
  RestNb = 0;
}

AstroObjs::AstroObjs(const AstroData& ad, AstroChart* ac, int ith, enum ChartSubType cst) : AstroData(ad), ChartParams(*ac->Cbp)
{
  Ac = ac;
  Ith = ith;
  Cst = cst;
  PlaChars = PLACHARS;
  Show = TRUE;
  Ar = Drs;
  RestNb = 0;
  DefSweFlags = 0;
  Recompute();
}

AstroObjs::~AstroObjs()
{
	~Dct;
}

/** Returns the string value of the objet index, i.e. "12Cap45" */
bool AstroObjs::GetValStr(int index, AstroString& as) const
{
  double a;
  int s, deg, min;
  as = "";
  if(index < Sun || index > Last_Star || !(*Ar == index))
  {
    as = *Asr->ObjError;
    return FALSE;
  }
  a = Vals.Get(index);
  s = (int)a / 30;
  a -= (double)(s * 30);
  HMR(a, deg, min);
  if(deg < 10)
    as = " ";
  as += deg;
  as += (*Asg < s);
  a = (double)min * 60.0 / 100.0 + ROUND;
  min = (int)a;
  if(min < 10)
    as += "0";
  as += min;
  as += Vals.Retro(index) ? " R" : "  ";
#ifndef KDE
  as += '\0';
#endif
  return TRUE;
}

/** Returns the object name */
const AstroString& AstroObjs::operator << (int i) const
{
  if(i < 0 || i > Last_Star)
    return *Asr->ObjError;
  if(i > Last_Planet && i < First_Exotic)
    return *Asr->ObjError;
  if(i > Last_Exotic && i < Ascendant)
    return *Asr->ObjError;
  if(i >= First_Star && i <= Last_Star)
    return *Asr->StarsNames[i - First_Star];
  if(i >= First_Exotic && i <= Last_Exotic)
    i = i - First_Exotic + 23;
  if(i >= Ascendant && i <= House_12)
    i = i - Ascendant + 31;
  return *Asr->ObjNames[i];
}

/** Returns the short name of the corresponding object */
const AstroString& AstroObjs::operator < (int i) const
{
  if(i < 0 || i > Last_Star)
    return *Asr->ObjError;
  if(i > Last_Planet && i < First_Exotic)
    return *Asr->ObjError;
  if(i > Last_Exotic && i < Ascendant)
    return *Asr->ObjError;
  if(i >= First_Star && i <= Last_Star)
    return *Asr->StarsShortNames[i - First_Star];
  if(i >= First_Exotic && i <= Last_Exotic)
    i = i - First_Exotic + 23;
  if(i >= Ascendant && i <= House_12)
    i = i - Ascendant + 31;
  return *Asr->ObjShortNames[i];
}

const AstroString& AstroObjs::GetSubTypeName() const
{
	if(Cst < NoSt || Cst > Last_SubType)
		return *Asr->SubTypesNames[NoSt];
	return *Asr->SubTypesNames[Cst];
}

const AstroString& AstroObjs::GetSubTypeShortName() const
{
	if(Cst < NoSt || Cst > Last_SubType)
		return *Asr->SubTypesShortNames[NoSt];
	return *Asr->SubTypesShortNames[Cst];
}

/** Returns the reference to a static AstroString
containing the text value (a.k.a "12Cap45")
of the given object index */
const AstroString& AstroObjs::operator % (int index) const
{
  static AstroString as;
  (void)GetValStr(index, as);
  return as;
}

enum Elements AstroObjs::GetObjElement(int index) const
{
  if(index >= Sun && index <= Last_Planet)
    return Asr->ElemPla[index];
  if(index >= Ascendant && index <= House_12)
    return Asr->ElemHouses[index - Ascendant];
  return Undefined_Element;
}

const AstroColor& AstroObjs::GetObjColor(int index) const
{
  if(index >= Chiron && index <= Vesta)
    return COL[Asr->AsteroColor];
  if(index >= First_Exotic && index <= Last_Exotic)
    return COL[Asr->ExoticColor];
  switch(GetObjElement(index))
  {
    case Fire  : return COL[Asr->FireColor];
    case Air   : return COL[Asr->AirColor];
    case Water : return COL[Asr->WaterColor];
    case Earth : return COL[Asr->EarthColor];
    default    : return COL[Asr->OtherColor];
  }
}

char AstroObjs::GetObjChar(int index) const
{
  if(index > (int)strlen(PlaChars))
    return 0;
  if(EuropUran && index == Uranus)
     return ALTURA;
  if(EuropPlut && index == Pluto)
     return ALTPLU;
  return PlaChars[index] != ' ' ? PlaChars[index] : 0;
}

void AstroObjs::CountFAWE(int* fire, int* air, int* water, int* earth) const
{
  int i;
  enum Elements e;
  *fire = *air = *water = *earth = 0;
  for(i = Sun; i < Last_Exotic; i++)
    switch(e = GetObjElement(i))
      {
        case Fire : (*fire)++;  break;
        case Air  : (*air)++;   break;
        case Water: (*water)++; break;
        case Earth: (*earth)++; break;
        default:;
      }
}

void AstroObjs::CountCfm(int *cardinal, int *fixed, int *mutab) const
{
  int i;
  enum Signs s;
  *cardinal = *fixed = *mutab = 0;
  for(i = Sun; i < Last_Exotic; i++)
    if(*this == i)
      switch(s = Asg->GetSign(*this * i))
      {
				case Aries:
				case Cancer:
        case Libra:
        case Capricorn:  (*cardinal)++; break;
        case Taurus:
        case Leo:
        case Scorpio:
        case Aquarius:   (*fixed)++; break;
        case Gemini:
        case Virgo:
        case Sagittarius:
        case Pisces:     (*mutab)++; break;
        default:;
      }
}

int AstroObjs::InHouse(int i, double *f) const
{
  //int j;
  if(!(*this == i)) return 0;
  return Vals.InHouse(i, f);
}

void AstroObjs::PutInHouse(int index, int house, double f)
{
  if(!(*this == index)) return;
  Vals.PutInHouse(index, house, f);
}

void AstroObjs::CountAsc(int *angular, int *succeedent, int *cadent) const
{
  int i, j;
  *angular = *succeedent = *cadent = 0;
  for(i = Sun; i < Last_Exotic; i++)
    if(*this == i)
      switch(j = InHouse(i, 0))
      {
        case 1:
        case 4:
        case 7:
        case 10: (*angular)++; break;
        case 2:
        case 5:
        case 8:
        case 11: (*succeedent)++; break;
        case 3:
        case 6:
        case 9:
        case 12: (*cadent)++; break;
      }
}

bool AstroObjs::Compute()
{
	Dc.julday = GetJd();
	Dc.ith = Ith;
	Dc.first = Cst > Single ? Ac->Acb->Search(1, AstroChartBase::Any, TRUE) : Ith;
	Dc.cst = Cst;
	Dc.latitude = Latitude.Get(TRUE);
	Dc.longitude = Longitude.Get(TRUE);
	Dc.extraflags = DefSweFlags;
	Dc.restr = RestNb;
	Dc.gethouses = TRUE;
	Dct = Dc;
	return Aif->Recompute(Dct);
}

void AstroObjs::SetSubChart(enum ChartSubType cst)
{
	Cst = cst;
	Dc.first = Cst > Single ? Ac->Acb->Search(1, AstroChartBase::Any, TRUE) : Ith;
}

bool AstroObjs::Recompute()
{
	if(Compute())
	{
		Aif->GetValues(Ith) > Vals;
		return TRUE;
	}
	AstroString as, buf;
	Aif->GetError(as);
	AstroMessage(Sprintf(i18n("Error: the computing server returned\n\n%a"), &buf, &as));
	return FALSE;
}
