/***************************************************************************
                          astroprocess.h  -  description
                             -------------------
    begin                : Fri May 11 2001
    copyright            : (C) 2001 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROPROCESS_H
#define ASTROPROCESS_H

#define ASTROPROCSESSERVER "skylendar-serv"
#define ASTROCOMPUTING     "skylendar-comp-"
#define ASTROINTERFACE     "skylendar-intf"
#define COMPUTEOBJECT      "compobj"
#define SCANCLIENT         "scanclient-"

#include "../config.h"
#include "astrodefs.h"
#include "astrotransfer.h"

#ifdef KDE
#include <dcopclient.h>
#include <dcopobject.h>
#include <qobject.h>
#include <qstring.h>
#endif

extern "C"
{
# include <time.h>
}

class AstroInterface;
class _AstroRestrictions;
class Values;
class AstroComputing;
class AspScanList;
class ScanInterface;

class ComputeSubChart
{
	public:

	ComputeSubChart(AstroComputing* ac) { Ac = ac; }
	virtual bool Compute(const struct DataComp& dc) const;

	protected:

	Values& V(int i) const;
	const _AstroRestrictions& Ar(int i) const;
	AstroComputing* Ac;
};

class ComputeSubChartList
{
	public:

	ComputeSubChartList();
	void Init(AstroComputing* ac);
	~ComputeSubChartList();

	const ComputeSubChart* Get(int i) const { return Cscl[i]; }

	private:

	ComputeSubChart* Cscl[Last_SubType];
};

#ifdef KDE

/** The computing class itself */
class AstroComputing: public QObject, /*virtual*/ public DCOPObject
{
  Q_OBJECT
  K_DCOP

 public:

 friend class ComputeSubChart;

  /** Constructor */
  AstroComputing();

  /** Destructor */
  ~AstroComputing();

 k_dcop:

  /**
   *  Sets the time and hour values. Time must NOT be expressed in decimal
   */
  void SetTZTime(double time, int day, int month, int year);

  /*
   * Returns the time offset (hour.minute)
   */
  double TimeOffset();

  /**
   * Says if the daylight time is active or not
   */
  bool Isdst();

  /** Sets (using a QString) the timezone file. if set == TRUE,
      the time conversion is set */
  void KDESetTZ(QString file, bool set = TRUE);

  /**
   * Sets (usign a QString) the swe data path
   */
  void KDESetSweDataPath(QString path);

  /** Sets all the AstroRestrictions to art
  @param art The art transfert object */
  void InitRestricts(ARTransfer art);

  /** Sets the AstroRestrictions i to art content */
 void SetRestrict(ARTransfer art, int i);

 /** Recomputes the objects values and puts results into slot i
 @param dct the transfer object
 @return TRUE if OK
 */
  bool Recompute(DCTransfert dct);

  /** Retrieves the values from the slot slot
  @param slot the slot number 0..NBDATA
  @return The VTransfer values transfer object */
  VTransfer GetValues(int slot) const;

  QString KDEGetError();

  /** Kills the current process */
  void Kill();

  /** Does nothing. Just for client/server syncro */
  void Sync();

  /** Scan all the aspects until julday */
  ASYNC ScanAsp(DCTransfert dct, double origjd, QString scserver, int scanflags);

  /** Stops at once Aspects scan */
  ASYNC StopScan();

#endif

 private:

  time_t tt;
  char tzsave[80];
  char serr[256];
  bool IfScan;
  _AstroRestrictions* Ar[NBDATA];
  VTransfer Vt[NBDATA];
  Values V[NBDATA];
  ComputeSubChartList Cscl;
  bool Compute(const struct DataComp&);
  void TestAsp(const _AstroAspects& aa, const AstroFoundAspectsTable& afat1,
	const AstroFoundAspectsTable& afat2, const Values& v1, const Values& v2,
	double j, const struct DataComp& dc, ScanInterface& sci);
};

/** This class manages a aspects scan receiver client */
class ScanClient: public QObject, public DCOPObject
{
  Q_OBJECT
  K_DCOP

public:

	friend class AspScanList;

  /** Constructor */
  ScanClient(AspScanList* ascl, int id);

  /** Destructor */
  ~ScanClient();

  /** If scan process finished */
  bool IfEnd() { bool e = End; if(e) End = FALSE; return e; }

k_dcop:

	/** Gets a found aspect
		@param i the ith natal object
		@param ipos ith object position. ipos < 0 = retro
		@param j the jth moving object
		@param jpos the object position, jpos < 0 = retro
		@param aspect the given aspect
		@param date the exact aspect date
	*/
	 ASYNC GetAspect(int i, double ipos, int j, double jpos, int aspect, double date);

	 /** Gets an object's direction change date
		@param i the ith moving object
		@param ipos the ith object position, ipos < 0 = retro
		@param date the direction change exact date
	*/
	 ASYNC GetRetro(int i, double ipos, double date);

	 /** Gets a progression percentage for the progression bar
		@param i the percentage
	*/
	 ASYNC Progress(int i);

	 /** Received when scan finished */
	 ASYNC EndScan();
 

private:

 AspScanList* Ascl;
 bool End;
};

/** This class manages the creation of the computing servers */
class AstroProcServer
{
 public:

  /** Creates a computing server and return its interface */
  friend AstroInterface* CreateInterface(bool cmpserver = TRUE);

  /** Constructor */
  AstroProcServer();

  /** Ends the application */
  static void KillAll();

 private:

  static int NewCmpServer(); 
  int pid;
};

#endif




