/***************************************************************************
                          astrorings.cpp  -  description
                             -------------------
    begin                : Sat Dec 30 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrorings.h"
#include "astrodefs.h"
#include "astrochartbase.h"
#include "astrostring.h"
#include "astroresources.h"
#include "astroaspects.h"

#include <cmath>

extern "C"
{
#include "swephexp.h"
#include "sweph.h"
};

#define PREV(x) ((x) ? (x) - 1 : NbSlots - 1)
#define NEXT(x) (((x) >= NbSlots - 1) ? 0 : (x) + 1)

// A table of sizes, numbers of slots and corresponding chars sizes
static const struct RingType
{
  int Thickness, NbSlots, MaxNbObj;
  AstroGraphics::SizeChars Sc;
}
  Sct[3] =
{
  { 15,  50,  28, AstroGraphics::Large  }, 
  { 18,  90,  61, AstroGraphics::Medium },
  { 28, 160,  82, AstroGraphics::Tiny   }
};

AstroRings::AstroRings(AstroGraphics *parent, int x, int y, int size, ChartParams* cp, AstroChartBase* acb, AstroFoundAspectsList& afal, AstroObjs* first, AstroObjs* second) : AstroGraphics(parent, x, y, size, size)
{
  int i, count = 0, height;
  AstroString test("0");
  Size = size;
  Acb = acb;
  Cp = cp;
  Afal = &afal;
  First = first;
  Second = second;
  //const AstroObjs& a = *ACB[Acb->Search(1, AstroChartBase::SingleOnly)];
  Angle0 = (/**a.*/First->IfHouses() && Cp->LeftAscendant) ? -DEGRAD(/*a*/ (*First) * Ascendant) : 0.0;
  Sv = 0;
  for(i = 0; i < NBDATA; i++)
    if(ACB == i && ACB[i]->Show)
      count += ACB[i]->CountObjs();
  RingX = RingY = 2 + (Size - 4) / 2;
  TextMetrics(test, &Size0, &height);
  SelectSizeClass(count);
  NbRings = Acb->Count();
  Narrow = NbRings > 2 ? TRUE : FALSE;
  init();
}

void AstroRings::SelectSizeClass(int i)
{
  int j = 0, nb;
  nb = Sct[j].MaxNbObj;
  nb += nb / 2;
  if(i > nb)
    j = 1;
  nb = Sct[1].MaxNbObj;
  nb += nb / 2;
  if(i > nb)
    j = 2;
  Thickness = Sct[j].Thickness;
  NbSlots   = Sct[j].NbSlots;
  Sc        = Sct[j].Sc;
  MaxNbObj  = Sct[j].MaxNbObj;
  Halfthick = Size / (2 * Thickness); 
  if(Sv)
    delete Sv;
  Sv = new struct Slot[NbSlots];
  for(j = 0; j < NbSlots; j++)
    Sv[j].IfSlot = Sv[j].Inc = FALSE;
}

void AstroRings::PushBack(int i)
{
  int s = i, j;
  while(Sv[s].IfSlot)
    s = PREV(s);
  Sv[s].IfSlot = TRUE; Sv[s].Inc = FALSE;
  for(j = s; j != i; j = NEXT(j))
    Sv[j] = Sv[NEXT(j)];
}

static AstroString Shorten(const AstroString& as, int level)
{
  static const int Max0LengthText[] = { 34, 28, 24 };
  int l = as.Len(), l1 = Max0LengthText[level - 1];
  return (l > l1) ? as.Left(l1) : as;
}

void AstroRings::Cuspide(double angle, int i, int i1)
{
  double v;
  int x, y, x1, y1;
  v = cos(angle);
  x = (int)(i1 * v);
  x1 = (int)(i * v);
  v = sin(angle);
  y = (int)(i1 * v);
  y1 = (int)(i * v);
  Move(RingX + x1, RingY - y1);
  Line(RingX + x, RingY - y);
}

void AstroRings::CornerText(const AstroString& as, int line, enum Corner Corner)
{
  int h, w, h1;
  TextMetrics(as, &w, &h);
  h = 4 * h / 5;
  h1 = Height - (2 * h / 5);
  if(line < 1 || line > 3) return;
  switch(Corner)
  {
    case Top_Left     : Move(Size0, line * h); Text(Shorten(as, line)); break;
    case Top_Right    : Move(Width - w, line * h); Text(Shorten(as, line)); break;
    case Bottom_Left  : Move(Size0, h1 - (--line * h)); Text(Shorten(as, line)); break;
    case Bottom_Right : Move(Width - w, h1 - (--line * h)); Text(Shorten(as, line));
  };
}

void AstroRings::CornerInfo(const AstroObjs&a, int i)
{
  AstroString as, as1;
  enum Corner c;
  switch(i)
  {
    case 0: c = Top_Right;    break;
    case 1: c = Top_Left;     break;
    case 2: c = Bottom_Left;  break;
    case 3: c = Bottom_Right; break;
    default: return;
  };
  FontSize(Tiny);
  if(c == Top_Right || c == Bottom_Right)
  		CornerText(Sprintf(i18n("%a: %d"), &as, &a.Name, i + 1), 1, c);
  else
  		CornerText(Sprintf(i18n("%d: %a"), &as, i + 1, &a.Name), 1, c);
  a.GetDate(as1);
  CornerText(Sprintf(i18n("%a %a %a"), &as, Asr->SubTypesShortNames[a.GetSubType()], &as1, &a.Get(as, HM_Time, TRUE)), 2, c);
  CornerText(a.PlaceName, 3, c);
}

void AstroRings::RingSigns()
{
  int i, x, y, x1, y1, rr = RingRadius, rs, thick;
  double c;
  const double a = TWOPI / 12;
  Fore();
  FontSize(Narrow ? Medium : Large, TRUE);
  thick = (Narrow ? 14 :  8) * Sct[0].Thickness / 10;
  RingRadius -= Size / thick;
  Circle(RingX, RingY, RingRadius);
  for(i = 1, c = Angle0; i <= 12; i++, c += a)
  {
    if(c > TWOPI) c -= TWOPI;
    Cuspide(c, rr, RingRadius);
  }
  rs = RingRadius + Size / (thick * 2);
  for(i = 0, c = Angle0 + TWOPI / 24 + PI; i < 12; i++, c += a)
  {
    if(c > TWOPI) c -= TWOPI;
    GlyphMetrics(Asg->GetChar((enum Signs) i, *Cp), &x1, &y1);
    x = (int)(rs * cos(c)) - x1 / 2;
    y = (int)(rs * sin(c)) - (y1 / 3);
    Move(RingX + x, RingY - y);
    Color(Asg->GetColor((enum Signs) i));
    Sign((enum Signs) i);
  }
}

void AstroRings::DegreesRing()
{
  int i, thick, rr = RingRadius, rs;
  double c;
  const double a = TWOPI / 360.0;
  thick = (Narrow ? 5 : 3) * Sct[0].Thickness;
  RingRadius -= Size / thick;
  rs = RingRadius + (rr - RingRadius) / 2;
  Fore();
  for(i = 1, c = Angle0; i <= 360; i++, c += a)
  {
    if(c > TWOPI) c -= TWOPI;
    Cuspide(c, rr, ((i -1) % 5) ? rs : RingRadius);
  }
  RingRadius = rs;
}

void AstroRings::HousesRing(const AstroObjs& a)
{
  int i, x, y, x1, y1, rr = RingRadius, thick;
  double c, rs;
  AstroString House;
  if(!a.IfHouses()) return;
  enum House h = (~a)->GetHouse();
  bool hide = (h == Vehlow || h == Azimuthal) ? FALSE : TRUE;
  thick = Narrow ? 28 : 12 * Sct[0].Thickness / 10;
  RingRadius -= Size / thick;
  
  rs = RingRadius + (rr - RingRadius) / 2;
  FontSize(Narrow ? Tiny : Medium);
  for(i = House_1; i <= House_12; i++)
  {
    c = Angle0 + DEGRAD(a * i) + PI;
    if(c > TWOPI) c -= TWOPI;
    Grey();
    Cuspide(c, rr, 0);
    if(!(hide && (i == House_1 || i == House_4 || i == House_7 || i == House_10)))
    {
      House = i - House_1 + 1;
      TextMetrics(House, &x1, &y1);
      x = (int)(rs * cos(c)) - x1/ 2;
      y = (int)(rs * sin(c)) - (y1 / 3);
      Move(RingX + x, RingY - y);
      Color(a.GetObjColor(i - House_1 + 1));
      Text(House);
    }
  }
  RingRadius = (int)rs;
}

void AstroRings::Insert(enum Objs o, double angle)
{
  struct Slot *s = new Slot, *ptr, *ptr1;
  unsigned int nb = Sl.Count();
  s->O = o;
  s->Angle = angle;
  if(nb == 0)
  {
    Sl.Append(s);
    return;
  }
  if(nb == 1)
  {
    if(Sl.First()->Angle > s->Angle)
      Sl.Prepend(s);
    else
      Sl.Append(s);
    return;
  }
  if(Sl.First()->Angle > s->Angle)
    Sl.Prepend(s); 
  else
  {
    for(ptr = Sl.First(), ptr1 = Sl.Next(); ptr1; ptr = ptr1, ptr1 = Sl.Next())
    {
      if(ptr->Angle < s->Angle && ptr1->Angle >= s->Angle)
      {
	Sl.Insert(s);
	return;
      }
    }
    Sl.Append(s);
  }
}

void AstroRings::PutRing(const AstroObjs& a, int dlinelen)
{
  int i, size;
  struct Slot *s, *ps;
  double slotangle, rs, lrs, lrs2;
  Sl.Clear();
  size = Size / Thickness;
  if(Thickness == Sct[1].Thickness)
    size = 9 * size / 10;
  RingRadius -= size;
  rs = RingRadius + Halfthick;
  lrs = RingRadius - Halfthick;
  lrs2 = lrs - (dlinelen * size);
  for(i = 0; i < NbSlots; i++)
    Sv[i].IfSlot = Sv[i].Inc = FALSE;
  for(i = Sun; i <= Last_Planet; i++)
    if(a == i)
      Insert((enum Objs) i, a * i);
  if(a.IfExotic())
    for(i = First_Exotic; i <= Last_Exotic; i++)
      if(a == i)
     	  Insert((enum Objs) i, a * i);
  if(a.IfHouses())
    for(i = Ascendant; i <= House_12; i++)
      if(a == i)
     	  Insert((enum Objs) i, a * i);
  if(a.IfStars())
    for(i = First_Star; i <= Last_Star; i++)
      if(a == i)
     	  Insert((enum Objs) i, a * i);
  for(s = Sl.First(); s; s = Sl.Next())
  {
    i = (int)(s->Angle * NbSlots / 360.0);
    ps = &Sv[i];
    slotangle = i * 360.0 / NbSlots;
    if(ps->IfSlot)
    {
      int j = i;
      while(Sv[j].IfSlot)
     	  j = NEXT(j);
      ps = &Sv[j];
      if(abs(j - i) > 3)
      {
       	int k = PREV(j);
       	const int absji = abs(j - i);
        bool push = TRUE;
       	while(Sv[k].IfSlot)
       	{
       	  if(abs((int)(Sv[k].Angle * NbSlots / 360.0) - k) > absji)
       	  {
       	    push = FALSE;
       	    break;
       	  }
       	  k = PREV(k);
       	}
       	if(push)
       	{
       	  j = PREV(j);
       	  PushBack(j);
       	  ps = &Sv[j];
       	}
      }
    }
    ps->IfSlot = TRUE;
    ps->Angle  = s->Angle;
    ps->O      = s->O;
  }
  for(i = 0; i < NbSlots; i++)
    if(Sv[i].IfSlot)
    {
      slotangle = i * 360.0 / NbSlots;
      if(!a.GetObjChar(Sv[i].O) && Sv[PREV(i)].IfSlot && !Sv[PREV(i)].Inc)
     	  Sv[i].Inc = TRUE;
      PutObj(a, &Sv[i], slotangle, rs, lrs, lrs2);
    }
}

void AstroRings::PutObj(const AstroObjs& a, const struct Slot *sl, double angle, double rs, double lrs, double lrs2)
{
  int x, y, lx1, ly1, lx2, ly2, xc, yc;
  double c, lc;
  char s;
  angle = DEGRAD(angle);
  c = Angle0 + PI + angle;
  if(c > TWOPI) c -= TWOPI;
  s = a.GetObjChar(sl->O);
  if(s)
  {
    FontSize(Sc, TRUE);
    GlyphMetrics(s, &xc, &yc);
    xc /= 2; yc /= 3;
    x = (int)(rs * cos(c)) - xc;
    y = (int)(rs * sin(c)) - yc;
    Move(RingX + x, RingY - y);
    Color(a.GetObjColor(sl->O));
    Glyph(s);
  }
  else
  {
    const AstroString& Obj = a < sl->O;
    double x1 = c;
    if(x1 >= PI)
      x1 -= PI;
    if(x1 >= (PI / 6.0) && x1 <= (5.0 * PI / 6.0) && sl->Inc)
      rs  -= Halfthick;
    FontSize((AstroGraphics::SizeChars)(Sc - 1));
    TextMetrics(Obj, &xc, &yc);
    xc /= 2; yc /= 3;
    x = (int)(rs * cos(c)) - xc;
    y = (int)(rs * sin(c)) - yc;
    Move(RingX + x, RingY - y);
    Color(a.GetObjColor(sl->O));
    Text(Obj);
  }
  lc = Angle0 + PI + DEGRAD(sl->Angle);
  if(lc > TWOPI) lc -= TWOPI;
  lx1 = (int)(RingRadius * cos(c));
  ly1 = (int)(RingRadius * sin(c));
  lx2 = (int)(lrs * cos(lc));
  ly2 = (int)(lrs * sin(lc));
  Grey();
  Move(RingX + lx1, RingY - ly1);
  Line(RingX + lx2, RingY - ly2);
  if(lrs2 && Cp->CentralRing)
  {
    lx2 = (int)(lrs2 * cos(lc));
    ly2 = (int)(lrs2 * sin(lc));
    Grey();
    Dash(3);
    Line(RingX + lx2, RingY - ly2);
    Dash(0);
  }
  Fore();
  DrawDot();
}

void AstroRings::AscMidH(const AstroObjs& a)
{
  int x, y, rs = Size / 2;
  double c;
  if(a == Ascendant)
  {
    c = Angle0 + PI + DEGRAD(a * Ascendant);
    if(c > TWOPI) c -= TWOPI;
    x = (int)(rs * cos(c));
    y = (int)(rs * sin(c));
    Fore();
    Move(RingX + x, RingY - y);
    Line(RingX - x, RingY + y);
  }
  if(a == MC)
  {
    c = Angle0 + PI + DEGRAD(a * MC);
    if(c > TWOPI) c -= TWOPI;
    x = (int)(rs * cos(c));
    y = (int)(rs * sin(c));
    Fore();
    Move(RingX + x, RingY - y);
    Line(RingX - x, RingY + y);
  }
}

void AstroRings::ObjectsRings()
{
  int i, rr, l = 0, size = Size / Thickness;
  if(Thickness == Sct[1].Thickness)
    size = 9 * size / 10;
   for(i = NBDATA -1; i; i--)
   	if(ACB[i] && ACB[i]->Show)
   	{
   		l = i;
   		break;
   	}
   for(i = 0; i < NBDATA; i++)
    if(ACB == i && ACB[i]->Show)
    {
       PutRing(*ACB[i], l - i);
       CornerInfo(*ACB[i], i);
       rr = RingRadius;
    }
  else
    RingRadius -= size;
  RingRadius = rr;
}

void AstroRings::Repaint()
{
  Sl.Clear();
  init();
  RingSigns();
  DegreesRing();
  HousesRing(*First);
  ObjectsRings();
  AscMidH(*First);
  if(Cp->CentralRing)
   Aspects(Second ? Second : First);
}

void AstroRings::Aspects(AstroObjs* ao)
{
  AspectFound* af;
  int x, y, x1, y1;
  double xa, xb;
  AstroAspects aa(~(*ao));
  RingRadius -= Size / Thickness / 2 + Thickness / 4;
  for(af = Afal->First(); af; af = Afal->Next())
  {
   Color(COL[aa.Color(af->Asp)]);
   xa = DEGRAD(af->Angle_a) + Angle0 + PI;
   if(xa > TWOPI) xa -= TWOPI;
   xb = DEGRAD(af->Angle_b) + Angle0 + PI;
   if(xb > TWOPI) xb -= TWOPI;
   x = (int)(RingRadius * cos(xa));
   y = (int)(RingRadius * sin(xa));
   x1 = (int)(RingRadius * cos(xb));
   y1 = (int)(RingRadius * sin(xb));
   Move(RingX + x, RingY - y);
   switch(af->GetAccuracy())
   {
     case Exact:
     case Good:   Dash(0); break;
     case Medium: Dash(2); break;
     case Poor:   Dash(3);
     default:;
   }
   Line(RingX + x1, RingY - y1);
  }
  Dash(0);
}

AstroTextFrame::AstroTextFrame(AstroGraphics* parent, int x, int y, int w, int h, ChartParams* cp, AstroChartBase* acb) :AstroGraphics(parent, x, y, w, h)
{
  int height;
  AstroString Test("0");
  Acb = acb;
  Cp = cp;
  Fore();
  FontSize(Tiny);
  TextMetrics(Test, &Size0, &height);
  CharHeight = (4 * height) / 5;
  OrigWidth = LineWidth  = w + (Cp->DisplaySeconds ? 3 : 2) * Size0;
  Init();
}

AstroSideBar::AstroSideBar(AstroGraphics *parent, int x, int y, int w, int h, ChartParams* cp, AstroChartBase* acb, AstroObjs* ao) : AstroTextFrame(parent, x, y, w, h, cp, acb)
{
  Ao = ao;
}

int AstroSideBar::GetWidth() const
{
  int i = Ao->CountObjs() + 24, j = i / MaxNbLines;
  if(i % MaxNbLines)
    j++;
  return j * LineWidth;
}

void AstroTextFrame::Init()
{
  int h = GetHeight();
  FontSize(Tiny, TRUE);
  FontSize(Tiny);
  XCurs = Size0; YCurs = 1;
  SetWidth(GetWidth());
  SetHeight(h);
  MaxNbLines = (h / CharHeight);
  NbWidth = 1;
  Move(XCurs, CharHeight);
}

void AstroTextFrame::TextLine(const AstroString as, bool newline)
{
  Text(as, TRUE);
  if(newline)
    NewLine();
}

void AstroTextFrame::NewLine()
{
  if(YCurs == MaxNbLines)
  {
    XCurs += LineWidth;
    YCurs = 1;
    Move(XCurs, CharHeight);
  }
  else
  {
    Y += CharHeight;
    Move(XCurs, Y - PosY);
    YCurs++;
  }
}

void AstroSideBar::Repaint()
{
  int i;
  Init();
  Fore();
  for(i = 0; i < NBDATA; i++)
     if(Ao == ACB[i])
       StdHeader(*Ao, i);
  NewLine();
  if(Ao->IfHouses())
  {
    for(i = 1; i <= 12; i++)
      HouseLine(*Ao, i);
    NewLine();
  }
  for(i = Sun; i <= East_Point; i++)
    ObjLine(*Ao, i);
  for(i = Ascendant; i <= Descendant; i++)
    ObjLine(*Ao, i);
  if(Ao->IfExotic())
    for(i = First_Exotic; i <= Last_Exotic; i++)
      ObjLine(*Ao, i);
  if(Ao->IfHouses())
    for(i = House_1; i <= House_12; i++)
      ObjLine(*Ao, i);
  if(Ao->IfStars())
    for(i = First_Star; i <= Last_Star; i++)
      ObjLine(*Ao, i);
  NewLine();
  Fawe(*Ao);
  NewLine();
  Cfm(*Ao);
  NewLine();
  Asc(*Ao);
  Grey();
}

void AstroSideBar::StdHeader(const AstroObjs& a, int i)
{
  AstroString as, hms;
  AstroHM hm = a.GetOffset();
  as = i + 1; as += ": "; as += (a.GetDaylight() ? i18n("DT") : i18n("ST")) + " ";
  hm.Get(hms);
  TextLine(as + hms + i18n(" GMT"));
  a.Longitude.Get(hms, HM_Longitude); as = hms  + " ";
  a.Latitude.Get(hms, HM_Latitude);
  TextLine(as + hms);
  TextLine(Sprintf(i18n("%a Houses"), &as, Asr->HousesSystemNames[(~a)->GetHouse()]));
  TextLine(*Asr->SiderealSystemNames[(~a)->Sidereal + 1] + i18n(" Geocentric"));
  TextLine((~a)->Name);
}

void AstroSideBar::HouseLine(const AstroObjs& a, int house)
{
  AstroString as;
  int hs = House_1 + house - 1, w, h;
  double angle = a * hs;
  if(house < 1 || house > 12) return;
  Sprintf(i18n("%d%a house:  "), &as, 12, Asr->NthAbbrev[11]);
  TextMetrics(as, &w, &h);
  Sprintf(i18n("%d%a house:  "), &as,  house, Asr->NthAbbrev[house - 1]);
  Color(a.GetObjColor(hs));
  JustifText(as, Right, w);
  Color(Asg->GetColor(Asg->GetSign(angle)));
  DegSignMin(DecToDeg(angle), FALSE, FALSE, Cp->DisplaySeconds);
  NewLine();
}

void AstroSideBar::ObjLine(const AstroObjs& a, int i)
{
  double v;
  AstroString as;
  char c;
  if(a == i)
  {
    Color(a.GetObjColor(i));
    c = a.GetObjChar(i);
    JustifGlyph(c ? c : ' ', Left, Size0 * 2);
    JustifText(a < i, Left, Size0 * 3);
    v = DecToDeg(a * i);
    Color(Asg->GetColor(Asg->GetSign((int)v)));
    DegSignMin(v, FALSE, a[i]->Retro, Cp->DisplaySeconds);
    if(!Cp->DisplaySeconds)
      TextLine("  ", FALSE);
    Grey();
    Hms(DecToDeg((*a[i])[1]), "");
    NewLine();
  }
}
void AstroSideBar::DrawLabelNb(const AstroString& label, int size, int nb)
{
  int w = Size0 * 2;
  AstroString as;
  as = nb;
  JustifText(label, Left, size);
  Text(":", TRUE);
  JustifText(as, Right, w);
  RMove(w / 2);
}

void AstroSideBar::Fawe(const AstroObjs& a)
{
  int fire, air, water, earth, w = (Size0 * 11) / 2;
  a.CountFAWE(&fire, &air, &water, &earth);
  Grey();
  DrawLabelNb(i18n("Fire"), w, fire);  
  DrawLabelNb(i18n("Air"), w, air);
  NewLine();
  DrawLabelNb(i18n("Water"), w, water);
  DrawLabelNb(i18n("Earth"), w, earth);
}

void AstroSideBar::Cfm(const AstroObjs& a)
{
  int cardinal, fixed, mutab, w = (Size0 * 38) / 10;
  a.CountCfm(&cardinal, &fixed, &mutab);
  Grey();
  DrawLabelNb(i18n("Car"), w, cardinal);
  DrawLabelNb(i18n("Fix"), w, fixed);
  DrawLabelNb(i18n("Mut"), w, mutab);
}

void AstroSideBar::Asc(const AstroObjs& a)
{
  int angular, succeedent, cadent, w = (Size0 * 38) / 10;
  a.CountAsc(&angular, &succeedent, &cadent);
  Grey();
  DrawLabelNb(i18n("Ang"), w, angular);
  DrawLabelNb(i18n("Suc"), w, succeedent);
  DrawLabelNb(i18n("Cad"), w, cadent);
}

AstroAspectsFrame::AstroAspectsFrame(AstroGraphics* parent, int x, int y, int w, int h, ChartParams* cp, AstroChartBase* acb, AstroFoundAspectsList* afal, AstroObjs* first, AstroObjs* second) : AstroTextFrame(parent, x, y, w, h, cp, acb)
{
  Afal = afal;
  First = first;
  Second = second;
  LineWidth = 17 * Size0;
}

int AstroAspectsFrame::GetHeight() const
{
 int count = Afal->Count(), l = GetWidth() / (17 * Size0), l1 = count / l;
 return CharHeight * (20 + count % l ? l1 + 1 : l1);
}

void AstroAspectsFrame::Repaint()
{
 AstroAspects aa(Second ? ~(*Second) : ~(*First));
 Init();
 AspectFound* af;
 Fore();
 Move(0, 0);
 Line(GetWidth(), 0);
 Move(XCurs, CharHeight);
 for(af = Afal->First(); af; af = Afal->Next())
   DrawAspectFound(*af, aa);
}

void AstroAspectsFrame::DrawAspectFound(const AspectFound& af, const AstroAspects& aa)
{
  AstroString as;
  char c;
  const AstroObjs& s = Second ? *Second : * First;
  Color(First->GetObjColor(af.Oa));
  if(c = First->GetObjChar(af.Oa))
   JustifGlyph(c, Center, Size0 * 4);
  else
    JustifText((*First) < af.Oa, Left, Size0 * 4);
  Color(COL[aa.Color(af.Asp)]);
  JustifGlyph(aa.GetAspChar(af.Asp), Center, Size0 * 2);
  Color(s.GetObjColor(af.Ob));
  if(c = s.GetObjChar(af.Ob))
    JustifGlyph(c, Center, Size0 * 4);
  else
     JustifText(s < af.Ob, Right, Size0 * 4);
   Fore();
   switch(af.AspectSide)
   {
         case Exact: as = "="; break;
         case Applying: as = "<"; break;
         case Separating: as = ">"; break;
         default:;
   };
   JustifText(as, Right, (Size0 * 3 ) / 2);
   RMove(-Size0);
   double da = (af.Asp == Parallel || af.Asp == Antiparallel ? af.DiffLong : DiffAngle(af.Angle_a, af.Angle_b));
   Hms(DecToDeg(fabs(da - (aa * af.Asp))), "");
   NewLine();
}
