/***************************************************************************
                          astrosql.cpp  -  description
                             -------------------
    begin                : Wed Nov 7 2001
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "astrodefs.h"
#include "astrosql.h"
#include "dblogin.h"
#include "astroresources.h"
#ifdef KDE
#include <klineedit.h>
#include <kpassdlg.h>
#endif
#include <cstdlib>
#include <cstring>

#define BUFSIZE 80

AstroSQL::AstroSQL()
{
#ifdef KDE
 bool login = FALSE;
 DbLogin dbl;
 Ok = FALSE;
 Sdb = Sdb->addDatabase("QPSQL7");
 if(!Sdb)
   return;
 while(!login)
 {
    if(dbl.exec() == QDialog::Accepted)
    {
      Sdb->setDatabaseName(dbl.DbName->text());
      Sdb->setUserName(dbl.UserName->text());
      Sdb->setPassword(dbl.Password->text());
      Sdb->setHostName(dbl.HostName->text());
      if(Sdb->open() && !Sdb->isOpenError())
      {
				login = TRUE;
				Asr->DBHost = dbl.HostName->text();
				Asr->DbName = dbl.DbName->text();
				Asr->DBUser = dbl.UserName->text();
      }
     }
     else
			 return;
  }
  Ok = TRUE;
#else
  char buf[80];
  Ok = TRUE;
  sprintf(buf, "dbname = %s user = %s", dbname.Data(), user.Data());
  conn = PQconnectdb(buf);
  if(!conn)
  {
    Ok = FALSE;
    return;
  }
  enum ConnStatusType cst = PQstatus(conn);
  if(cst != CONNECTION_OK)
  Ok = FALSE;
#endif
}

/*int AstroRequest::Number()
{
#ifdef KDE
 if(!Ok || !isValid())
   return 0;
 int n, pos = at();
 last();
 n = at() + 1;
 seek(pos);
 return n;
#else
  return Ok ? PQntuples(Result) : 0;
#endif
}*/

/**int AstroRequest::GetIndex(const char *c)
{
#ifdef KDE
  return Ok ? Sr.position(c) : -1;
#else
  return Ok ? PQfnumber(Result, c) : -1;
#endif
}*/

bool AstroRequest::SetComment(const AstroString& as, Index idx)
{
#ifndef KDE
  enum ExecStatusType est;
#endif
  AstroString r("UPDATE Data SET Comment = '");
  r += as.Utf8();
  r += "' WHERE Idx = ";
  r += (int)idx;
  Ok = TRUE;
  Clear();
#ifdef KDE
  *SC(QSqlQuery*, this) = Asql->Sdb->exec(r);
  if(!isValid())
  {
    Ok = FALSE;
    return Ok;
  }
  Sr = Asql->Sdb->record(*this);
  first();
#else
  Result = PQexec(Asql->conn, r.Utf8().Data());
  if(!Result)
  {
    Ok = FALSE;
    return FALSE;
  }
  est = PQresultStatus(Result);
  if(est == PGRES_FATAL_ERROR || est == PGRES_NONFATAL_ERROR || est == PGRES_BAD_RESPONSE)
  {
    Ok = FALSE;
    return FALSE;
  }
#endif
  tuple = 0;
  return TRUE;
}

/**char* AstroRequest::Get(Index idx)
{
  if((int)idx == -1)
  {
    printf("Invalid index\n");
    return 0;
  }
#ifdef KDE
  static QVariant Qv;
  Qv = value(idx);
  const char *d = (const char *)Qv.asCString();
  return Ok ? (char *)d : 0;
#else
  return Ok ? PQgetvalue(Result, tuple, idx) : 0;
#endif
}*/

AstroString AstroRequest::operator < (const char* c)
{
  AstroString as;
#ifdef KDE
  as.Utf8(value(Sr.position(c)).asString().data());
#else
  as.Utf8(Get(GetIndex(c)));
#endif
  return as;
}

int AstroRequest::GetInt(const char *c)
{
#ifdef KDE
  return value(Sr.position(c)).asInt();
#else
  char buf[20];
  strncpy(buf, Get(GetIndex(c)), 20);
  return atoi(buf);
#endif
}

double AstroRequest::GetDouble(const char *c)
{
#ifdef KDE
  return value(Sr.position(c)).asDouble();
#else
  char buf[20];
  strncpy(buf, Get(GetIndex(c)), 20);
  return atof(buf);
#endif
}

char AstroRequest::GetChar(const char *c)
{
#ifdef KDE
  QString s = value(Sr.position(c)).asString().left(1);
  return (char)*s.data();
#else
  char buf;
  strncpy(&buf, Get(GetIndex(c)), 1);
  return buf;
#endif
}

bool AstroRequest::GetBool(const char *c)
{
#ifdef KDE
  return value(Sr.position(c)).asBool();
#else
  char buf;
  strncpy(&buf, Get(GetIndex(c)), 1);
  return buf == 't' ? TRUE : FALSE;
#endif
}

AstroSQL::~AstroSQL()
{
#ifdef KDE
  Sdb->close();
#else
  PQfinish(conn);
#endif
}

bool AstroRequest::Exec(const char *c, ...)
{
#ifdef KDE
#else
  enum ExecStatusType est;
#endif
  AstroString v, *aptr;
  char buf[2048], buf1[40], *ptr, *ptr1, **x = (char **)&c, *y;
  int i;
  double *fptr;
  x++;
  for(ptr = (char *)c, ptr1 = buf; *ptr; ptr++)
    if(*ptr != '%')
    {
    	if(*ptr == '\\')
    		ptr++;
      *ptr1++ = *ptr;
     }
    else
    {
      ptr++;
      switch(*ptr)
      {
        case 'c' : *ptr1++ = (int)*x;
	  x++;
	  break;
        case 's' : y = (char *)*x;
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
        case 'd' : i = (int)*x;
	  y = buf1;
	  sprintf(buf1, "%d", i);
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
        case 'b': i = (int)*x;
	  y = buf1;
	  sprintf(buf1, "%s", i == 0 ? "\'f\'" : "\'t\'");
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
        case 'f' : fptr = (double *)x;
	y = buf1;
	sprintf(buf1, "%f", *fptr);
	  while(*y)
	    *ptr1++ = *y++;
	  x += 2;
	  break;
	  case 'a': aptr = (AstroString *)*x;
	  v = aptr->Utf8();
	  y = (char *)v.Data();
	  while(*y)
	    *ptr1++ = *y++;
	  x++;
	  break;
      };
    }
  *ptr1 = 0;
  Ok = TRUE;
  Clear();
  //printf("%s\n", buf);
#ifdef KDE
  *SC(QSqlQuery*, this) = Asql->Sdb->exec(buf);
  if(lastError().number() != -1)
  {
    Ok = FALSE;
    printf("err = %s\n", lastError().databaseText().data());
    return Ok;
  }
  first();
  Sr = Asql->Sdb->record(*this);
#else
  Result = PQexec(Asql->conn, buf);
  if(!Result)
  {
    Ok = FALSE;
    return FALSE;
  }
  est = PQresultStatus(Result);
  if(est == PGRES_FATAL_ERROR || est == PGRES_NONFATAL_ERROR || est == PGRES_BAD_RESPONSE)
  {
    Ok = FALSE;
    return FALSE;
  }
#endif
  tuple = 0;
  return TRUE;
}

void AstroRequest::First()
{
#ifdef KDE
 first();
#endif
}

bool AstroRequest::Next()
{
#ifdef KDE
 return next();
#endif
 tuple++;
}

bool AstroRequest::Valid()
{
#ifdef KDE
 return isValid();
#endif
}

Index AstroRequest::NextId(const char *table)
{
#ifdef KDE
  AstroString as = "SELECT NEXTVAL('";
  as += table;
  as += "_Idx_Seq')";
  QSqlQuery qs = Asql->Sdb->exec(as);
  if(qs.lastError().number() != -1)
  {
    return 0;
  }
  qs.first();
  return qs.value(0).asInt();
#endif
}

/*Index AstroRequest::GetINSId()
{
  return PQoidValue(Result);
}*/
