/***************************************************************************
                          astrotransfer.h  -  description
                             -------------------
    begin                : Mon May 6 2002
    copyright            : (C) 2002 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROTRANSFER_H
#define ASTROTRANSFER_H

#include <dcopclient.h>
#include "astrorestrictions.h"

/**This class is a templated class for the
data transfer client<->server
  *@author Christophe Gros
  */

template <class Pa> class AstroTransfer: public QByteArray
{
public:

	/** Constructor */
  inline AstroTransfer()
  {
    IsSet = FALSE;
  }

  	/** Copy operator */
  inline void operator = (const AstroTransfer &at)
  {
    QByteArray::operator = (at);
    IsSet = FALSE;
  }

  /** Constructor with data */
  inline AstroTransfer(const Pa &p)
  {
    setRawData((const char*)&p, sizeof(Pa));
    IsSet = TRUE;
  }

  	/** Setting with data (for setting)*/
  inline void operator = (const Pa& p)
  {
    if(IsSet)
      resetRawData(data(), sizeof(Pa));
    setRawData((const char*)&p, sizeof(Pa));
    IsSet = TRUE;
  }

  	/** Soft setting with data (for retrieving) */
  inline void operator << (const Pa& p)
  {
    if(!IsSet)
      assign((const char*)&p, sizeof(Pa));
  }

  /** Don't forget to use this operator when finished with this object */
  inline void operator~()
  {
     resetRawData(data(), sizeof(Pa));
  }

  	/** Retrieves data */
  inline void operator > (Pa& p)
  {
    const char *c = data();
    if(c) p = *((Pa*)c);
    if(IsSet)
    {
      resetRawData(data(), sizeof(Pa));
      IsSet = FALSE;
    }
  }

 private:

  bool IsSet;
};

/* A structure containing all the data needed by the computation */
struct DataComp
{
	double julday, latitude, longitude;
	enum ChartSubType cst;
	int ith, first, extraflags, restr;
	bool gethouses;
};

/* structure containing the 6 values returned by the swe computation routines */
struct DataValues
{
  double V[6];
  bool Retro;
  inline double operator[](int i) const { return V[i]; }
};

/** A class that manages all the values for the computed objects */
class Values
{
 public:

	double Julday;

  /** returns the ith data */
  inline const DataValues& operator[](int i) const { return Data[i]; }

  /** Returns the first (longitude) value for the ith object */
  inline double Get(int i) const { return Data[i].V[0]; }

  /** If the ith object is retro */
  inline bool Retro(int i) const { return Data[i].Retro; }

  /** Gets the jth value of the ith object */
  inline double Get(int i, int j) const { return Data[i].V[j]; }

  /** Sets all the values for the ith object */
  void Set(int i, double d, bool r = FALSE, double d1 = 0.0, double d2 = 0.0, double d3 = 0.0, double d4 = 0.0, double d5 = 0.0);

  /** Just sets the first value of object i */
  void CSet(int i, double d, bool r = FALSE);

  /** Returns the number of the house where the object index dwells in */
  int InHouse(int index, double *f = 0) const;

  /** Puts the object index in house at position f
	@param index the object index.
	@param house the house number 1..12
	@param f position within the house 0.0..1.0
	*/
	void PutInHouse(int index, int house, double f);

 private:

  DataValues Data[Last_Star + 1];
};

typedef AstroTransfer<struct DataComp> DCTransfert;
typedef AstroTransfer<_AstroRestrictions> ARTransfer;
typedef AstroTransfer<Values> VTransfer;
#endif
