/***************************************************************************
                          astrowindow.cpp  -  description
                             -------------------
    begin                : Sat Dec 30 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrowindow.h"
#include "stdwheelchart.h"
#include "astroresources.h"
#include "astroprocess.h"
#include "astrointerface.h"
#include "astromainwindow.h"
#include "astrorestrictions.h"
#include "astrosql.h"

#ifdef KDE
#include <qpainter.h>
#include <qpen.h>
#include <qfont.h>
#include <kprinter.h>
#include <qpicture.h>
#include <qworkspace.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qdragobject.h>
#include <qpaintdevicemetrics.h>
#include <qregexp.h>
#include <qapplication.h>
#endif

extern AstroMainWindow* Amw;

AstroWindow::AstroWindow(const AstroMainWindow* amw, AstroData* ad1, enum ChartType ect, AstroData* ad2, AstroData* ad3, AstroData* ad4): QScrollView(amw->Qws), AstroGraphicChart(Asr->Width, Asr->Height)
{
  int vsbw, hsbh;
  AstroString as;
  setAcceptDrops(TRUE); 
  QTim = 0;
	ct = ect;
  Day = Month = Year = Hour = 0;
  Minute = Rate = 1;
  MilliSec = 1000;
  Backward = FALSE;
  Jbt = Minutes;
  SetData(ad1, 0);
  if(ad2)
    SetData(ad2, 1);
  if(ad3)
    SetData(ad3, 2);
  if(ad4)
    SetData(ad4, 3);
  Width = GetWidth();
  Height = GetHeight();
#ifdef KDE
  resizeContents(Width, Height);
  Qw = viewport();
  Qw->setMinimumSize(100, 100);
  Qw->setSizeIncrement(Width / 5, Height / 5);
  Cbp->Qw = Qw;
  vsbw = 30;//Asv.verticalScrollBar()->width();
  hsbh = 30; //Asv.horizontalScrollBar()->height();
  emit resize(Width + vsbw, Height + hsbh);
  Qp = new QPainter;
  Pen = new QPen;
  astro = new QFont(ASTROFONT);
  CHECK_PTR(astro);
  text = new QFont(TEXTFONT);
	CHECK_PTR(text);
	QPict = new QPicture;
	ParamPm = new QPopupMenu(this);
  ParamPm->insertItem(i18n("Dynamically Resize"), 1);
	ParamPm->insertItem(i18n("Central Aspects Ring"), 2);
	ParamPm->insertItem(i18n("Aspects Frame"), 3);
	ParamPm->insertItem(i18n("Displays Seconds"), 4);
	ParamPm->insertItem(i18n("Displays Mid-Points"), 5);
	ParamPm->insertItem(i18n("Ascendant on the Left"), 6);
	ParamPm->insertItem(i18n("Side Bars"), 7);
  GetTitle(as);
  setCaption(as);
  connect(this, SIGNAL(Close(AstroWindow *)), amw, SLOT(Close(AstroWindow *)));
	connect(ParamPm, SIGNAL(activated(int)), this, SLOT(Activated(int)));
	Repaint();
  show();
#endif
}

AstroWindow::~AstroWindow()
{
}

void AstroWindow::closeEvent(QCloseEvent* e)
{
	emit Close(this);
  QWidget::closeEvent(e);
}

#ifdef KDE
void AstroWindow::drawContentsOffset(QPainter *p, int ox, int oy, int cx, int cy, int cw, int ch )
{
  p->translate((float)-ox, (float)-oy);
  Redraw(p);
}

void AstroWindow::resizeEvent(QResizeEvent* qre)
{
  if(Cbp->DynamicResize)
  {
    int nw = (int)((double)OrigWidth * (double)qre->size().width() / (double)qre->oldSize().width()), nh = (int)((double)nw * (double)OrigHeight / (double)OrigWidth);
    Resize(nw, nh);
    OrigWidth = nw;
    OrigHeight = nh;
    Repaint(TRUE);
    resizeContents(GetWidth(), GetHeight());
  }
  QScrollView::resizeEvent(qre);
}
 
void AstroWindow::dragEnterEvent(QDragEnterEvent* event)
{
   event->accept(QTextDrag::canDecode(event));
}

void AstroWindow::dropEvent(QDropEvent* event)
{
  QString text, as("^"), as1;
  ButtonType bt = Undefined_Button;
  if(Acb->Running)
  {
		qApp->beep();
		return;
  }
  if(QTextDrag::decode(event, text))
  {
    int i;
    as += PACKAGE;
    QRegExp test(as);
    if(test.match(text) != -1)
    {
      test = as + " Data:";
      if(test.match(text) != -1) bt = DataButton;
      test = as + " Restrict:";
      if(test.match(text) != -1) bt = RestrictButton;
      test = as + " Type:";
      if(test.match(text) != -1) bt = TypeButton;
      test = as + " Subtype:";
      if(test.match(text) != -1) bt = SubTypeButton;
      if(bt != Undefined_Button)
      {
			text = text.right(text.length() - text.find(':') - 1);
			i = text.toInt();
			switch(bt)
			{
     			case DataButton:
	    			SetData(Adl->Get(i), Amw->Ring - 1); break;
	  			case RestrictButton:
	   		    SetRestrict(Arl->Get(i), Amw->Ring - 1); break;
	        case TypeButton:
	        	ManageTypes(i); break;
  	  		case SubTypeButton:
	    			ManageSubTypes(i); break;
	     }
			 Repaint(TRUE);
      }
    }
  }
}
#endif

void AstroWindow::ManageTypes(int i)
{
	enum ChartType cti = (enum ChartType)i;
	if(cti < StdWheels || cti >= Unknown_ChartType)
		return;
	if(cti == AspScList && Acb->Count() < 2)
	{
		AstroMessage(i18n("Sorry, this chart needs 2 visible rings."));
		return;
	}
	ct = cti;
}

void AstroWindow::ManageSubTypes(int i)
{
  AstroString as;
  int r = Amw->Ring -1;
  bool createnew;
  Index oldidx;
  Index idx;
  switch(i)
  {
    case Show_Hide:
      if(!SetSubType(Show_Hide, r))
     	  AstroMessage(i18n("Sorry, Can't hide this ring !"));
      break;
    case ExtrnSave:
      if(!(idx = ExtractandSave(r, oldidx, createnew, Acb->GetAspectsList())))
        AstroMessage(i18n("Can't extract this ring !"));
      else
      {
        if(oldidx >= BOUND && !createnew)
        {
          Adl->Get(oldidx)->Idx = idx;
          Amw->Aiw->UpdateDataIdx(oldidx, idx);
        }
        if(createnew)
         (void)new AstroIconsWindow::AstroIconButton(Amw->Aiw, Adl->Get(idx));
      }
      break;
    case Single:
    case Transit:
    case Direction:
    case Progression:
      if(!SetSubType((enum ChartSubType)i, r))
     	  AstroMessage(Sprintf(i18n("Sorry, Can't set this subtype ring to %a"), &as, Asr->SubTypesNames[i]));
      break;
    default:;
  }
}

AstroGraphicChart::AstroGraphicChart(int width, int height): AstroGraphics(0, 0, 0, width, height), AstroChart()
{
 Cbp->Ag = Base = this;
 ct = StdWheels;
 Dm = GraphicMode;
 DblBuf = 0;
 Kp = 0;
 Width  = OrigWidth = width;
 Height = OrigHeight = height;
 PosX = PosY = 0;
 Size = Width / 50;
}

AstroPrintPage::AstroPrintPage(AstroString& title, AstroChartBase* acb, AstroGraphicChart* agc)
{
	Acb = acb;
	Agc = agc;
	Page = 1;
	Title = &title;
	connect(acb, SIGNAL(NewPage()), this, SLOT(NewPage()));
}

AstroPrintPage::~AstroPrintPage()
{
	disconnect(Acb, SIGNAL(NewPage()), this, SLOT(NewPage()));
}

void AstroPrintPage::NewPage()
{
	
	Footer();
	Agc->Kp->newPage();
	Page++;
}

void AstroPrintPage::Header()
{
}

void AstroPrintPage::Footer()
{
	int w = Agc->GetWidth();
	AstroString l = PACKAGE, r;
	l += " ";
	l += VERSION;
	Sprintf(i18n("Page %d"), &r, Page);
	Agc->Move(0, Agc->GetHeight());
	Agc->JustifText(Page % 2 ? l : r, AstroGraphics::Left, w / 3);
	Agc->JustifText(*Title, AstroGraphics::Center, w / 3);
	Agc->JustifText(Page % 2 ? r : l, AstroGraphics::Right, w / 3);
}

void AstroGraphicChart::SetDisplayMode(enum DisplayMode dm)
{
  Dm = dm;
  switch(Dm)
  {
    case GraphicMode:
      if(DblBuf) { delete DblBuf; DblBuf = 0; }
      if(Kp)     { delete Kp; Kp = 0; }
			Acb->IfPage = FALSE;
      break;
    case DblBufMode:
      if(Kp)     { delete Kp; Kp = 0; }
			Acb->IfPage = FALSE;
      DblBuf = new QPixmap(Width, Height);
      CHECK_PTR(DblBuf);
      DblBuf->setOptimization(QPixmap::BestOptim);
      break;
    case PrintMode:
      if(DblBuf) { delete DblBuf; DblBuf = 0; }
      Kp = new KPrinter;
      CHECK_PTR(Kp);
      if(!Kp->setup())
      {
       	Dm = GraphicMode;
       	delete Kp;
       	Kp = 0;
      }
			if(Acb->ct >= GraphicList)
				Acb->IfPage = TRUE;
  };
}

void AstroGraphicChart::BackgroundColor()
{
#ifdef KDE
  Qw->setBackgroundColor(*COL.Back);
#endif
}

void AstroGraphicChart::Repaint(bool b)
{
  int w, h;
  AstroChart::Repaint(b);
#ifdef KDE
	QFont timefont, *timefptr;
  switch(Dm)
  {
    case  GraphicMode:
      delete QPict;
      QPict = new QPicture;
      Qp->begin(QPict);
      BackgroundColor();
      Display();
      Qp->end();
			P;
      break;
    case DblBufMode:
      DblBuf->fill(*COL.Back);
      Qp->begin(DblBuf);
      Display();
      Qp->end();
			P;
      break;
    case PrintMode:
			if(Acb->IfPage)
			{
				AstroString as;
				GetTitle(as);
				AstroPrintPage app(as, Acb, this);
				PrinterSize(w, h);
				Resize(w, h);
				Qp->begin(Kp);
				Display();
				app.Footer();
				Qp->end();
				P;
				Resize(OrigWidth, OrigHeight);
			}
			else
			{
				int w1, h1;
				w = GetWidth();
				h = GetHeight();
				w1 = (OrigWidth - OrigWidth / 8) * PRINTZOOMFACT;
				h1 = (OrigHeight - OrigHeight / 8) * PRINTZOOMFACT;
				Resize(w1, h1);
				// Temporary change
				timefont.setRawName("-URW-Nimbus Roman No9 L-medium-r-normal--0-0-0-0-p-0-iso8859-1");
				timefptr = text;
				text = &timefont;
				Qp->begin(Kp);
				QPen& pn = (QPen&)Qp->pen();
				pn.setWidth(BOLDLINE);
				ResizeView(w, h);
				Qp->setWindow(0, 0, w * PRINTZOOMFACT, h * PRINTZOOMFACT);
				Display();
				Qp->end();
				P;
				text = timefptr;
				Resize(OrigWidth, OrigHeight);
			}
      break;
  };
#endif
}

void AstroGraphicChart::PrinterSize(int&w, int &h)
{
	QPaintDeviceMetrics qpd(Kp);
  w = qpd.width();
	h = qpd.height();
}

void AstroGraphicChart::ResizeView(int w, int h)
{
  QPaintDeviceMetrics qpd(Kp);
  int rw = qpd.width();
  Qp->setViewport(0, 0, rw, (int)(((double)rw * (double)h) / (double)w));
}

void AstroWindow::Repaint(bool b)
{
  AstroString as;
  switch(Dm)
  {
    case GraphicMode:
      SetWidth(Asr->Width);
      SetHeight(Asr->Height);
      AstroGraphicChart::Repaint(b);
      resizeContents(GetWidth(), GetHeight());
      viewport()->update();
      GetTitle(as);
      setCaption(as);
      break;
    case DblBufMode:
      AstroGraphicChart::Repaint(b);
      bitBlt(viewport(), 0, 0, DblBuf, 0, 0, viewport()->width(),
	      viewport()->height(), CopyROP, TRUE);
      break;
    case PrintMode:
      AstroGraphicChart::Repaint(TRUE);
  };
}

void AstroWindow::Anim(bool start)
{
  if(start)
  {
    SetDisplayMode(DblBufMode);
    Acb->Running = TRUE;
    QTim = new QTimer;
    CHECK_PTR(QTim);
    connect(QTim, SIGNAL(timeout()), this, SLOT(TimerSlot()));
    QTim->start(MilliSec, FALSE);
  }
  else
  {
    QTim->stop();
    Acb->Running = FALSE;
    SetDisplayMode(GraphicMode);
    delete QTim;
    QTim = 0;
  }
}

void AstroWindow::TimerSlot()
{
  int s = Backward ? -1 : 1;
  AstroObjs& ao = *(*this)[Amw->Ring - 1];
  ao.Modified = TRUE;
  ao.Name = "";
  ao.AstroDate::Recompute();
  double j = IncTime(ao.GetJd(), s * Minute, s * Hour, s * Day, s * Month, s * Year);
  ao.SetJd(j);
  if(!ao.Corrected)
		ao.Recompute();
  Repaint();
}

void AstroWindow::mousePressEvent(QMouseEvent *e) 
{
	switch(e->button())
	{
		case RightButton:
			 ParamPm->setItemChecked(1, Cbp->DynamicResize);
			 ParamPm->setItemChecked(2, Cbp->CentralRing);
			 ParamPm->setItemChecked(3, Cbp->AspectsFrame);
			 ParamPm->setItemChecked(4, Cbp->DisplaySeconds);
			 ParamPm->setItemChecked(5, Cbp->DisplayMidPoints);
			 ParamPm->setItemChecked(6, Cbp->LeftAscendant);
			 ParamPm->setItemChecked(7, Cbp->Sidebars);
			 ParamPm->exec(e->pos()); break;
		default:;
	};
}

void AstroWindow::Activated(int i)
{
	if(Acb->Running)
  {
		qApp->beep();
		return;
  }
	switch(i)
	{
		case 1: Cbp->DynamicResize    = !Cbp->DynamicResize; break;
		case 2: Cbp->CentralRing      = !Cbp->CentralRing; break;
		case 3: Cbp->AspectsFrame     = !Cbp->AspectsFrame; break;
		case 4: Cbp->DisplaySeconds   = !Cbp->DisplaySeconds; break;
		case 5: Cbp->DisplayMidPoints = !Cbp->DisplayMidPoints; break;
		case 6: Cbp->LeftAscendant    = !Cbp->LeftAscendant; break;
		case 7: Cbp->Sidebars         = !Cbp->Sidebars; break;
	};
	Repaint(TRUE);
}

void AstroWindow::SetJbt(enum JumpBaseType jbt)
{
  Jbt = jbt;
  Day = Month = Year = Hour = Minute = 0;
  switch(Jbt)
  {
    case Minutes:   Minute = Rate; break;
    case Hours:     Hour   = Rate; break;
    case Days:      Day    = Rate; break;
    case Months:    Month  = Rate; break;
    case Years:     Year   = Rate; break;
    case Centuries: Year   = 100 * Rate; break;
    case Millenia:  Year   = 1000 * Rate; break;
  };
}

void AstroWindow::SetRate(int r)
{
  Rate = r;
  switch(Jbt)
  {
    case Minutes:   Minute = Rate; break;
    case Hours:     Hour   = Rate; break;
    case Days:      Day    = Rate; break;
    case Months:    Month  = Rate; break;
    case Years:     Year   = Rate; break;
    case Centuries: Year   = 100 * Rate; break;
    case Millenia:  Year   = 1000 * Rate; break;
  };
}

AstroIconsList::AstroIconsList()
{
  AstroIcon* ai;
  AstroString name, file;
  AstroRequest Arq;
  if(!Arq.Exec("SELECT * FROM TypeIcons"))
  {
    AstroMessage(i18n("Can't load Type icons"));
    exit(1);		 
  }
  
  name = i18n("Now chart");
  ai = new AstroIcon("now.png", &name);
  Append(ai);

  if(Arq.Valid()) do
  {
    name = Arq < "Name";
    file = Arq < "File";
    ai = new AstroIcon(file.Data(), &name);
    Append(ai);
  }
  while(Arq.Next());
}

AstroIconsWindow::AstroIconsWindow(AstroMainWindow *amw) : QListView(amw->centralWidget())
{
  int i;
  Idx = 0;
  FirstXY = IsPressed = FALSE;
  Aib = 0;
  const AstroPixmap& gp = *Asr->Pixmaps[0];
#ifdef KDE
  setBackgroundMode( QWidget::PaletteBase );
  setRootIsDecorated(TRUE);
  addColumn(i18n("Data"));
  Data         = new QListViewItem(this, i18n("Data sets"));
  Restrictions = new QListViewItem(this, i18n("Restrictions sets"));
  Types        = new QListViewItem(this, i18n("Types"));
  SubTypes     = new QListViewItem(this, i18n("Subtypes"));
  Data->setOpen(TRUE);
  Data->setPixmap(0, gp);
  Restrictions->setPixmap(0, gp);
  Types->setPixmap(0, gp);
  SubTypes->setOpen(TRUE);
  SubTypes->setPixmap(0, gp);
  DataPm = new QPopupMenu(this);
  DataPm->insertItem(i18n("Edit"), 1);
  DataPm->insertItem(i18n("Save"), 2);
  DataPm->insertItem(i18n("Delete"), 3);
  DataPm->insertItem(i18n("Duplicate"), 4);
  RestPm = new QPopupMenu(this);
  RestPm->insertItem(i18n("Edit"), 1);
  RestPm->insertItem(i18n("Save"), 2);
  RestPm->insertItem(i18n("Delete"), 3);
  RestPm->insertItem(i18n("Duplicate"), 4);
  connect(this, SIGNAL(clicked(QListViewItem*)), SLOT(Clicked(QListViewItem*)));
  connect(this, SIGNAL(mouseButtonPressed(int, QListViewItem*, const QPoint&, int)), SLOT(Pressed(int, QListViewItem*, const QPoint&, int)));
  connect(this, SIGNAL(doubleClicked(QListViewItem*)), SLOT(DoubleClicked(QListViewItem*)));
  connect(DataPm, SIGNAL(activated(int)), this, SLOT(Activated(int)));
  connect(RestPm, SIGNAL(activated(int)), this, SLOT(Activated(int)));
#endif
  for(i = Show_Hide; i <= Last_SubType; i++)
    (void)new AstroIconButton(this, i);
  (void)new AstroIconButton(this, Adl->First());
  (void)new AstroIconButton(this, Drs);
  for(i = StdWheels; i < Unknown_ChartType; i++)
  	(void)new AstroIconButton(this, i, TypeButton);
}

void AstroIconsWindow::AddData(const AstroData* ad)
{
  Adl->AddData((AstroData*)ad);
  (void)new AstroIconButton(this, ad);
}

void AstroIconsWindow::AddRestrict(const AstroRestrictions* ar)
{
  Arl->AddRestrictions((AstroRestrictions*)ar);
  (void)new AstroIconButton(this, ar);
}

void AstroIconsWindow::DeleteCurrent()
{
  switch(Aib->Bt)
  {
    case DataButton:
    	if(Idx != 0)
    	{
      	Data->takeItem(Aib);
      	Adl->RemoveFromDb(Idx);
      	delete Aib;
  			Aib = 0;
        Idx = 0;
      }
      else
      	AstroMessage(i18n("Can't remove this icon"));
      break;
    case RestrictButton:
    	if(Idx != 1)
    	{
      	Restrictions->takeItem(Aib);
      	Arl->RemoveFromDb(Idx);
      	delete Aib;
  			Aib = 0;
      }
      else
			AstroMessage(i18n("Can't remove this icon"));
      break;
    default:
      AstroMessage(i18n("Can't remove this icon."));
  }
}


void AstroIconsWindow::Clicked(QListViewItem *qlvi)
{
  IsPressed = FALSE;
  const AstroIconButton* aib = SC(AstroIconButton *, qlvi);
  if(aib)
    Idx = aib->Idx;
}

void AstroIconsWindow::UpdateDataIdx(Index oldidx, Index newidx)
{
  AstroIconButton* aib;
  if(Aib && Aib->Bt == DataButton && Aib->Idx == oldidx)
    Idx = Aib->Idx = newidx;
  aib = GetDataButton(oldidx);
  if(aib)
     aib->Idx = newidx;
  if(oldidx != newidx)
    Ahl->ChangeIdx(oldidx, newidx);
}

AstroIconsWindow::AstroIconButton::AstroIconButton(AstroIconsWindow* parent, const AstroData* ad) : QListViewItem(parent->Data)
{
  Idx = ad->Idx;
  Bt = DataButton;
#ifdef KDE
  setText(0, ad->Name);
  setPixmap(0, *(*Ail)[ad->ChartType]);
#endif
}

AstroIconsWindow::AstroIconButton::AstroIconButton(AstroIconsWindow* parent, const AstroRestrictions* ar) : QListViewItem(parent->Restrictions)
{
  Idx = ar->Idx;
  Bt = RestrictButton;
#ifdef KDE
  setText(0, ar->Name);
  setPixmap(0, *Asr->Pixmaps[1]);
#endif
}

AstroIconsWindow::AstroIconButton::AstroIconButton(AstroIconsWindow* parent, int index, ButtonType bt) : QListViewItem(bt == SubTypeButton ? parent->SubTypes : (bt == TypeButton ? parent->Types : 0))
{
  Idx = index;
  Bt = bt;
  switch(Bt)
  {
    case SubTypeButton:
#ifdef KDE
      setText(0, *Asr->SubTypesNames[index]);
      setPixmap(0, *Asr->Pixmaps[1]);
      break;
#endif
		case TypeButton:
			setText(0, *Asr->TypesNames[index - 1]);
			setPixmap(0, *Asr->Pixmaps[index + 11]);
      break; 
    default:;
  };
}

AstroIconsWindow::AstroIconButton* AstroIconsWindow::GetDataButton(Index idx) const
{
   QListViewItem* qlvi = Data->firstChild();
   AstroIconButton* aib;
   while(qlvi)
   {
     aib = SC(AstroIconsWindow::AstroIconButton*, qlvi);
     if(aib->Idx == idx) return aib;
     qlvi = qlvi->nextSibling();
   }
   return 0;
}

void AstroIconsWindow::Pressed(int b, QListViewItem* item, const QPoint& qp, int c)
{
	AstroIconButton* aib = SC(AstroIconButton*, item);
  Aib = (item != Data && item != SubTypes) ? aib : 0;
	switch(b)
	{
		case 1: FirstXY = IsPressed = TRUE;
  			break;
    case 2:
			if(Aib)
			{
				Idx = Aib->Idx;
				switch(aib->Bt)
				{
					case DataButton: DataPm->exec(qp); break;
					case RestrictButton: RestPm->exec(qp); break;
					default:;
				};
			}
			break;
		}
}

void AstroIconsWindow::ChangeCurrent(const AstroData* ad)
{
  AstroIconButton* aib = SC(AstroIconButton*, currentItem());
#ifdef KDE
  aib->setText(0, ad->Name);
  aib->setPixmap(0,*(*Ail)[ad->ChartType]);
#endif 
}

void AstroIconsWindow::ChangeCurrent(const AstroRestrictions* ar)
{
  AstroIconButton* aib = SC(AstroIconButton*, currentItem());
#ifdef KDE
  emit aib->setText(0, ar->Name);
#endif
}

void AstroIconsWindow::contentsMouseMoveEvent(QMouseEvent * e)
{
  if(!Aib || !IsPressed) return;
  if(FirstXY)
  {
    FirstXY = FALSE;
    X = e->x();
    Y = e->y();
  }
  else
  {
    int x = abs(X - e->x()), y = abs(Y - e->y());
    if(x > 5 || y > 5)
    {
      AstroString as(PACKAGE);
      switch(Aib->Bt)
      {
        case DataButton:     as += " Data:";     as += Aib->Idx; break;
        case RestrictButton: as += " Restrict:"; as += Aib->Idx; break;
        case TypeButton:     as += " Type:";     as += Aib->Idx; break;
        case SubTypeButton:  as += " Subtype:";  as += Aib->Idx; break;
        default:;
      };
      Idx = Aib->Idx;
#ifdef KDE
      QDragObject *drobj = new QTextDrag(as, this);
      drobj->dragCopy();
#endif
      IsPressed = FALSE;
    }
  }
} 

void AstroIconsWindow::Activated(int i)
{
	switch(i)
	{
		case 1: Amw->Edit(); break;
		case 2: Amw->Save(); break;
		case 3: Amw->Delete(); break;
		case 4: Amw->Duplicate(); break;
	}
}



void AstroIconsWindow::DoubleClicked(QListViewItem* qlvi)
{
	AstroIconButton* aib = SC(AstroIconButton*, qlvi);
	if(aib)
	{
		Idx = aib->Idx;
		Amw->Edit();
	}
}
