/***************************************************************************
                          graphiclistbase.cpp  -  description
                             -------------------
    begin                : mer ao 21 2002
    copyright            : (C) 2002 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "graphiclistbase.h"

static const struct
{
  enum AstroGraphics::SizeChars Sc; //{ Very_Tiny, Tiny, Medium, Large, Huge };
  enum AstroGraphics::Justif J; //  { Left, Right, Center, Dot };
} Styles[] =
{
  { AstroGraphics::Tiny,   AstroGraphics::Left }, // PlainText
  { AstroGraphics::Large,  AstroGraphics::Center }, // MainTitle
  { AstroGraphics::Medium, AstroGraphics::Center }, // Tilte
  { AstroGraphics::Tiny,   AstroGraphics::Left }, // TabText
	{ AstroGraphics::Tiny,   AstroGraphics::Right}, // TabRight
	{ AstroGraphics::Tiny,   AstroGraphics::Dot} // TabDot
};

GraphicListBase::GraphicListBase(ChartBaseParams* cbp, ObjsVector* ov) : GraphicChartBase(cbp, ov, cbp->Ag)
{
	Init();
	cm = GraphicMode;
	DualChart = FALSE;
}

void GraphicListBase::Init()
{
	TabList = 0;
	Tab = 0;
	Ts = PlainText;
	XMargin = Ag->GetWidth() / 20;
	YMargin = Ag->GetHeight() / 20;
	W = Ag->GetWidth() - 2 * XMargin;
	H = Ag->GetHeight() - 2 * YMargin;
	Ag->Move(XMargin, YMargin);
}

GraphicListBase::~GraphicListBase()
{
}

const AstroString& GraphicListBase::Dbl(AstroString& as, double d, bool four, bool sci)
{
	char buf[12];
	if(four)
		sprintf(buf, sci ? "%.4e" : "%.4f", d);
	else
		sprintf(buf, sci ? "%.2e" : "%.2f", d);
	as = buf;
	return as;
}

void GraphicListBase::SetTextStyle(enum TextStyle ts)
{
	Ts = ts;
	Ag->FontSize(Styles[Ts].Sc);
	Ag->FontSize(Styles[Ts].Sc, TRUE);
}

void GraphicListBase::DrawText(const AstroString& as)
{
	int w, h;
	Ag->TextMetrics(as, &w, &h);
	Ag->JustifText(as, Styles[Ts].J, Ag->GetWidth());
}

void GraphicListBase::DrawTabText(const AstroString& as)
{
	int w1, h, l = TabList[Tab + 1] - TabList[Tab];
	Ag->TextMetrics(" 000", &w1, &h);
	Ag->JustifText(as, Styles[Ts].J, l, w1);
	Tab++;
}

bool GraphicListBase::NewLine()
{
	int w, h, l;
	Tab = 0;
	Ag->TextMetrics(" ", &w, &h);
	l = Ag->Y + h;
	if(l >= H)
	{
		if(IfPage)
		{
			emit NewPage();
			l = YMargin;
		}
		else
			SetHeight(l);
	}
	Ag->Move(XMargin, l);
	return TRUE;	
}

