/***************************************************************************
                          graphiclistbase.h  -  description
                             -------------------
    begin                : mer ao 21 2002
    copyright            : (C) 2002 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHICLISTBASE_H
#define GRAPHICLISTBASE_H

#include "astrographics.h"
#include "astrochartbase.h"

/**This class implements what is necessary for
the tabulated lists of values, arrays etc...
  *@author Christophe Gros
  */

class GraphicListBase : public GraphicChartBase
{

public:

 	/** Constructor */ 
	GraphicListBase(ChartBaseParams*, ObjsVector*);

	/** Destructor */
	~GraphicListBase();

	/** Different paragraph styles */
	enum TextStyle { PlainText, MainTitle, Title, TabText, TabRight, TabDot};
	
protected:

	void Init();

	/** Sets the style of the coming text */
	void SetTextStyle(enum TextStyle ts);

	/** Draw text */
	void DrawText(const AstroString&);

	/** Draw tabbed text, i.e. The cursor is moved to the next tab stop */
	void DrawTabText(const AstroString&);

	/** Sets the tab position list
	@param t a pointer to an array of int containing positives values for the tab */
	inline void SetTabList(const int *t) { TabList = (int *)t; }

	/** New line */
	bool NewLine();

	/** Displays a double number
	@param as the buffer, also returned by Dbl
	@param d the double number itself
	@param four if TRUE, four digits, else only two
	@param sci if TRUE scientific notation
	@return buffer with the ASCII value of d
	*/
	const AstroString& Dbl(AstroString& as, double d, bool four = FALSE, bool sci = FALSE);
	int W, H;
	
private:

	enum TextStyle Ts;
  int *TabList, Tab, XMargin, YMargin;
};

#endif
