BEGIN;

CREATE TABLE Data -- The Data table
(
	Idx Int4 UNIQUE NOT NULL, -- Unique Index
	Name VARCHAR   NOT NULL,
	BirthDay DATE  NOT NULL,
	BirthTime TIME NOT NULL,
	Gender CHAR    NOT NULL, -- 'M' = male, 'F' = female, 'n' = neutral
	ChartType Int2 DEFAULT 1, -- 0 = Beginning 1 = End, etc...
	Daylight BOOL  DEFAULT 'f', --  If daylight time saving
	TimeZone NUMERIC(5, 2) DEFAULT '0.0',
	Comment TEXT, -- Free text
	PlaceIdx Int4  NOT NULL -- Place row index
);

CREATE SEQUENCE Data_Idx_Seq;

-- CREATE INDEX Data_Idx_Key ON Data(Idx);

CREATE INDEX Data_Name ON Data (Name) ;

INSERT INTO Data (Idx, Name, BirthDay, BirthTime, Gender, ChartType, PlaceIdx)
VALUES (NEXTVAL('Data_Idx_Seq'), 'Default Data', '1-1-2000', '12:00', 'N', 1, 1);

CREATE TABLE Place -- The places table
(
	Idx Int4 UNIQUE NOT NULL, -- Row index
	Name VARCHAR, -- Place name
	Latitude NUMERIC(5, 2)  NOT NULL, -- Latitude in decimal
	Longitude NUMERIC(5, 2) NOT NULL, -- Longitude in decimal
	CountryIdx Int4 DEFAULT 1 -- Index of relevant country
);

CREATE SEQUENCE Place_Idx_Seq;

-- CREATE INDEX Place_Idx_Key ON Place(Idx);

CREATE INDEX Place_Name ON Place (Name) ;

INSERT INTO Place (Idx, Name, Latitude, Longitude, CountryIdx)
VALUES (NEXTVAL('Place_Idx_Seq'), 'Default Place', 0.0, 0.0, 1) ; -- The default place

CREATE TABLE Country -- The countries table
(
	Idx Int4 UNIQUE NOT NULL, -- Unique index
	Name VARCHAR   NOT NULL, -- Country name
	TZFile VARCHAR NOT NULL -- Zone info file name
);

CREATE SEQUENCE Country_Idx_Seq;

-- CREATE INDEX Country_Idx_Key ON Country(Idx);

CREATE INDEX Country_Name ON Country (Name) ;

INSERT INTO Country (Idx, Name, TZFile) VALUES(NEXTVAL('Country_Idx_Seq'), 'Default Country', 'GMT') ;

CREATE TABLE ObjRest -- Restrictions data for each object i.e. planet, star, etc...
(
	Index Int4            NOT NULL, -- Restrictions set index
	ObjIndex Int2         NOT NULL, -- Object index
	Rest Bool             NOT NULL, -- If not visible
	OrbAdd  NUMERIC(4, 2) NOT NULL, -- Orb addition
	Influence Int2        NOT NULL -- Power of object
);

CREATE INDEX ObjRest_Index ON ObjRest (Index);

CREATE TABLE AspRest -- Restrictions data for each aspect
(
	Index Int4        NOT NULL, -- Restrictions index
	AspIndex Int2     NOT NULL, -- Aspect index
	Rest Bool         NOT NULL, -- If not visible
	Orb NUMERIC(5, 2) NOT NULL, -- Orb
	Color Int2        NOT NULL, -- Color 0..15
	Influence Int2    NOT NULL -- Power
);

CREATE INDEX AspRest_Index ON AspRest (Index);

CREATE TABLE Restrictions -- The restrictions table
(
	Idx Int4 UNIQUE NOT NULL, -- Restrictions unique index
	Name VARCHAR   UNIQUE NOT NULL, -- NAme
	Houses Bool    NOT NULL, -- If houses used
	Exotic Bool    NOT NULL, -- If Exotic object used e.g. Apollo, etc...
	Stars Bool     NOT NULL, -- If stars used
	Minor Bool     NOT NULL, -- If Minor aspects used .e.g. Sesquiquadrate
	HousesSys Int2 NOT NULL, -- If houses system code .e.g. Placidus, etc...
	TrueNode Bool  NOT NULL, -- If true lunar node
	Sidereal Int2  NOT NULL -- If Tropical or sidereal zodiac, and which one.
);

CREATE SEQUENCE Restrictions_Idx_Seq;

-- CREATE INDEX Restrictions_Idx_Key ON Restrictions(Idx);

CREATE INDEX Restrictions_Name ON Restrictions (Name);

CREATE TABLE TypeIcons 
(
	Idx Int4 UNIQUE NOT NULL,
	Name VARCHAR NOT NULL,
	File VARCHAR NOT NULL
);

CREATE SEQUENCE TypeIcons_Idx_Seq;

-- CREATE INDEX TypeIcons_Idx_key ON TypeIcons(Idx);

INSERT INTO TypeIcons (Idx, Name, file) VALUES(NEXTVAL('TypeIcons_Idx_Seq'), 'Birth/Beginning', 'beginning.png');
INSERT INTO TypeIcons (Idx, Name, file) VALUES(NEXTVAL('TypeIcons_Idx_Seq'), 'Death/end', 'end.png');
COMMIT;
