#include "searchdata.h"
#include "astroparams.h"
#include "astroutils.h"
#include <qbuttongroup.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include "astromainwindow.h"

extern AstroMainWindow* Amw;

/* 
 *  Constructs a SearchData which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
SearchData::SearchData()
    : _SearchData(0, 0, TRUE, 0)
{
  int i;
  SQLReqType = 0;
  RequestType->setButton(SQLReqType);
  ShowBtn(FALSE);
  DataLIST->setMultiSelection(TRUE);
  for(i = Sun; i < OBJNB -1; i++)
    Planets->insertItem(*Asr->ObjNames[i], i);
  for(i = 0; i < NBSTARS; i++)
   Planets->insertItem(*Asr->StarsNames[i], i + OBJNB -1);
  for(i = Conjunction; i < Unknown_Aspect; i++)
     Aspects->insertItem(*Asr->AspectNames[i], i);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
SearchData::~SearchData()
{
    // no need to delete child widgets MoveBackward, Qt does it all for us
}

/* 
 * public slot
 */
void SearchData::ClickedSlot(QListViewItem*)
{ 
}
/* 
 * public slot
 */
void SearchData::OkSlot()
{
  SearchItem* sa = SC(SearchItem*, DataLIST->firstChild());
  for(; sa; sa = SC(SearchItem*, sa->nextSibling()))
  {
    if(sa->isSelected() && !Adl->Get(sa->Idx))
    {
      AstroData* ad = new AstroData;
      CHECK_PTR(ad);
      ad->DbFetch(sa->Idx);
      Amw->Aiw->AddData(ad);
    }
  }
  emit accept();
}
/* 
 * public slot
 */
void SearchData::ReqTypeSlot(int i)
{
  SQLReqType = i;
  ShowBtn(i == 2 ? TRUE : FALSE);
}
/* 
 * public slot
 */
void SearchData::SearchSlot()
{
  int i;
  QString qs;
  char buf[3 * BUFSIZE], *t = "Data";
  switch(SQLReqType)
  {
    case 0:
  	  if(ReqLine->text().isEmpty())
   	    strcpy(buf,  "Data.Name LIKE '\\%' ");
      else
        snprintf(buf, 3 * BUFSIZE, "Data.Name ~* '%s'", (const char *)ReqLine->text().utf8());
      break;
    case 2:
      t = "VDV";
    case 1:
      strncpy(buf, (const char *)ReqLine->text().utf8(), 3 * BUFSIZE);
      break;
  };
  if(!Arq.Exec("SELECT %s.Name AS DName, %s.Idx AS Dix, BirthDay, ChartType, Place.Name AS PName FROM %s, Place WHERE %s.PlaceIdx = Place.Idx AND %s ORDER BY %s.Name", t, t, t, t, buf, t))
  {
    AstroMessage(i18n("Can't fetch data !"));
    return;
  }
  DataLIST->clear();
  i = 0;
  qs.setNum(i);
  emit Count->setText(qs);
  if(Arq.Valid()) do
  {
    (void)new SearchItem(DataLIST, Arq.GetInt("Dix"), Arq < "Dname", Arq < "BirthDay", Arq < "PName", *(*Ail)[Arq.GetInt("ChartType")]);
     qs.setNum(++i);
    emit Count->setText(qs);
  }
  while(Arq.Next());
}
/* 
 * public slot
 */
void SearchData::CancelSlot()
{
  emit reject();
}

SearchItem::SearchItem(QListView* parent, Index idx, const AstroString& name, const AstroString& date, const AstroString& place, const AstroIcon& icon) : QListViewItem (parent, name, date, place)
{
  Idx = idx;
  setPixmap(0, icon);
}

void SearchData::Insert(const QString& qs, int pos)
{
  int paf, idf, pat, idt;
  ReqLine->getSelection(&paf, &idf, &pat, &idt);
  emit ReqLine->insertAt(qs, paf, idf);
  if(pos)
     //emit ReqLine->moveCursor(QTextEdit::MoveBackward, FALSE);
     emit ReqLine->setCursorPosition(paf, idf - pos );
}

void SearchData::ANDSlot()
{
  Insert("AND ");
}

void SearchData::AspectsSlot(int i)
{
  QString qs("\'");
  qs += *Asr->AspectShortNames[i];
  qs += "\', ";
  Insert(qs);
}

void SearchData::PlanetsSlot(int i)
{
  QString qs("\'");
  qs += (i > OBJNB - 2) ? *Asr->StarsShortNames[i - OBJNB + 1] : *Asr->ObjShortNames[i];
  qs += "\', ";
  Insert(qs);
}


void SearchData::NOTSlot()
{
  Insert("NOT ");
}

void SearchData::ORSlot()
{
  Insert("OR ");
}

void SearchData::TAspSlot()
{
  Insert("TAsp(DataIdx, ) ", -2);
}

void SearchData::THouseSlot()
{
  Insert("THouse(DataIdx, ) ", -2);
}

void SearchData::TObjSlot()
{
  Insert("TObj(DataIdx, ) ", -2);
}

void SearchData::TRestSlot()
{
  Insert("TRest(DataIdx, ) ", -2);
}

void SearchData::ShowBtn(bool b)
{
  Planets->setEnabled(b);
  Aspects->setEnabled(b);
  TObjButton->setEnabled(b);
  TAspButton->setEnabled(b);
  RestButton->setEnabled(b);
  THouseButton->setEnabled(b);
  ANDButton->setEnabled(b);
  ORButton->setEnabled(b);
  NOTButton->setEnabled(b);
}
