#include <qlineedit.h>
#include "searchrest.h"
#include "astrorestrictions.h"
#include "astromainwindow.h"

extern AstroMainWindow* Amw;

/* 
 *  Constructs a SearchRest which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
SearchRest::SearchRest()
    : _SearchRest(0, 0, TRUE, 0)
{
  RestList->setMultiSelection(TRUE);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
SearchRest::~SearchRest()
{
    // no need to delete child widgets, Qt does it all for us
}

/* 
 * public slot
 */
void SearchRest::CancelSlot()
{
  emit reject();
}
/* 
 * public slot
 */
void SearchRest::Clicked(QListViewItem* qlvi)
{
 
}
/* 
 * public slot
 */
void SearchRest::OkSlot()
{
  SearchRestItem* sri = SC(SearchRestItem*, RestList->firstChild());
  for(; sri; sri = SC(SearchRestItem*, sri->nextSibling()))
  {
    if(sri->isSelected() && !Arl->Get(sri->Idx))
    {
      AstroRestrictions* ar = new AstroRestrictions;
      CHECK_PTR(ar);
      ar->DbFetch(sri->Idx);
      Amw->Aiw->AddRestrict(ar);
    }
  }
  emit accept();
}
/* 
 * public slot
 */
void SearchRest::SearchSlot()
{
  bool b;
  if(ReqLine->text().isEmpty())
  	b = Arq.Exec("SELECT Name, Idx FROM Restrictions WHERE Name LIKE '\\%' ORDER BY Name");
  else
  	b = Arq.Exec("SELECT Name, Idx FROM Restrictions WHERE Name ~* '%s' ORDER BY Name", (const char *)ReqLine->text().utf8());
	if(!b)
  {
    AstroMessage(i18n("Can't fetch restrictions !"));
    return;
  }
  RestList->clear();
  if(Arq.Valid()) do
    (void)new SearchRestItem(RestList, Arq.GetInt("Idx"), Arq < "Name");
  while(Arq.Next());
}

SearchRestItem::SearchRestItem(QListView* parent, Index idx, const AstroString& name) : QListViewItem (parent, name)
{
  Idx = idx;
}
