/***************************************************************************
                          stdgraphiclist.cpp  -  description
                             -------------------
    begin                : mer ao 21 2002
    copyright            : (C) 2002 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stdgraphiclist.h"
#include "astroutils.h"

#define DTT(d, four, sci) DrawTabText(Dbl(buf, d, four, sci))

StdGraphicList::StdGraphicList(ChartBaseParams* cbp, ObjsVector* ov) : GraphicListBase(cbp, ov)
{
	ct = GraphicList;
	//Redisplay();
}

StdGraphicList::~StdGraphicList()
{
}

void StdGraphicList::DisplayValues(const AstroString& as, const struct DataValues& dv)
{
	AstroString buf;
	SetTextStyle(TabText);
	DrawTabText(as);
	DrawTabText(dv.Retro ? "R" : " ");
	SetTextStyle(TabDot);
	DTT(DecToDeg(dv[0]), TRUE, FALSE);
	DTT(DecToDeg(dv[1]), TRUE, FALSE);
	DTT(dv[2], FALSE, TRUE);
	DTT(dv[3], FALSE, TRUE);
	DTT(dv[4], FALSE, TRUE);
	DTT(dv[5], FALSE, TRUE);
	NewLine();
}

bool StdGraphicList::Redisplay()
{
	int fl, fl1, s;
	AstroString as;
	First = (*this)[Search(1, Any, TRUE)];
  s = Search(2, Any, TRUE);
  Second = (s != -1) ? (*this)[s] : 0;
	fl = First->DefSweFlags;
	First->DefSweFlags |= SEFLG_SPEED;
	First->Recompute();
	if(Second)
	{
		fl1 = Second->DefSweFlags;
		Second->DefSweFlags |= SEFLG_SPEED;
		Second->Recompute();
	}
	Init();
	DisplayValues(*First);
	if(Second)
		DisplayValues(*Second);
	NewLine();
	if(Cbp->AspectsFrame)
	{
		SetTextStyle(Title);
		DrawText(i18n("Aspects List for"));
		NewLine();
		GetTitle(as);
		DrawText(as);
		NewLine();
		DisplayAspects();
		NewLine();
	}
	First->DefSweFlags = fl;
	if(Second)
		Second->DefSweFlags = fl1;
	return TRUE;
}

void StdGraphicList::DispMidPoints(const AstroObjs& a, const AstroObjs& b, bool ifsameplanet)
{
	int i, l = W / 6, l2 = l / 2;
	const int tab[] = { 0, l / 3, l + l2, 2 * l + l2, 3 * l + l2, 4 * l + l2 };
	AstroString as, buf;
	Afal.ComputeMidP(a, b, ifsameplanet);
	SetTabList(tab);
	SetTextStyle(TabText);
	DrawTabText("");
	DrawTabText(i18n("Object 1"));
	DrawTabText(i18n("Object 2"));
	DrawTabText(i18n("Angle"));
	NewLine();
	NewLine();
	for(i = 0; i < Afal.NbMP; i++)
	{
		const struct MidPoint& mp = Afal * i;
		as = i + 1;
		as += " ";
		SetTextStyle(TabRight);
		DrawTabText(as);
		SetTextStyle(TabText);
		DrawTabText(a << mp.Oa);
		DrawTabText(b << mp.Ob);
		Ag->DegSignMin(DecToDeg(mp.Mp), FALSE, FALSE, Cbp->DisplaySeconds);
		NewLine();
	}
}

void StdGraphicList::DisplayValues(const AstroObjs& ao)
{
	int i, l = W / 8;
	const int tab[] = { 0, l + l / 3, 2 * l, 3 * l, 4 * l, 5 *l, 6 * l, 7 * l, 8 * l, 9 * l };
	AstroString as, as1, as2, ty;
	NewLine();
	SetTabList(tab);
	SetTextStyle(Title);
	ty = ao.GetSubTypeName();
	DrawText(Sprintf(i18n("Standard %a Values List for"), &as, &ty));
	NewLine();
	ao.GetDate(as1);
	ao.GetTime(as2);
	Sprintf(i18n("%a (%a, %a, %a)"), &as, &ao.Name, &as1, &as2, &ao.PlaceName);
	DrawText(as);
	NewLine();
	SetTextStyle(TabText);
	DrawTabText(i18n("Name"));
	DrawTabText(i18n("Retro"));
	DrawTabText(i18n("Longitude"));
	DrawTabText(i18n("Latitude"));
	DrawTabText(i18n("Dist (AU)"));
	DrawTabText(i18n("Speed Lon"));
	DrawTabText(i18n("Speed Lat"));
	DrawTabText(i18n("Speed Dis"));
	NewLine();
	NewLine();
	for(i = 0; i <= Last_Planet; i++)
		if(ao == i)
			DisplayValues(ao << i, *ao[i]);
	if(ao.IfHouses())
		for(i = Ascendant; i <= House_12; i++)
			if(ao == i)
				DisplayValues(ao << i, *ao[i]);
	if(ao.IfExotic())
		for(i = First_Exotic; i <= Last_Exotic; i++)
			if(ao == i)
				DisplayValues(ao << i, *ao[i]);
	if(ao.IfStars())
		for(i = First_Star; i <= Last_Star; i++)
			if(ao == i)
				DisplayValues(ao << i, *ao[i]);
	NewLine();
	if(Cbp->DisplayMidPoints)
	{
		SetTextStyle(Title);
		DrawText(Sprintf(i18n("Mid-Points List for %a"), &as, &ao.Name));
		NewLine();
		DispMidPoints(ao, ao, FALSE);
	}
}

void StdGraphicList::DisplayAspects()
{
	int i, l = W / 6, l2 = l / 2;
	const int tab[] = { 0, l / 3, l + l2, 2 * l + l2, 3 * l + l2, 4 * l + l2, 5 * l + l2, 6 * l + l2 };
	AstroString as, buf;
	AstroAspects aa(Second ? ~(*Second) : ~(*First));
	AspectFound* af;
	const AstroObjs &ao = *First, &ao1 = Second ? * Second : * First;
	SetTabList(tab);
	Afal.Clear();
  if(Second && !(First->GetSubType() == Single && Second->GetSubType() == Single))
    Afal.GetList(aa, *Second, *First, TRUE);
  else
   Afal.GetList(aa, *First, *First, FALSE);
	SetTextStyle(TabText);
	DrawTabText(i18n(""));
	DrawTabText(i18n("Object 1"));
	DrawTabText(i18n("Aspect"));
	DrawTabText(i18n("Object 2"));
	DrawTabText(i18n("Actual Orb"));
	DrawTabText(i18n("Direction"));
	NewLine();
	NewLine();
	for(i = 1, af = Afal.First(); af; i++, af = Afal.Next())
	{
		as = i;
		as += " ";
		SetTextStyle(TabRight);
		DrawTabText(as);
		SetTextStyle(TabText);
		DrawTabText(ao << af->Oa);
		DrawTabText(aa << af->Asp);
		DrawTabText(ao1 << af->Ob);
		SetTextStyle(TabDot);
		DTT(DecToDeg(DiffAngle(af->Angle_a, af->Angle_b)), FALSE, FALSE);
		switch(af->AspectSide)
		{
			case Exact:      as = i18n("Exact"); break;
			case Applying:   as = i18n("Applying"); break;
			case Separating: as = i18n("Separating"); break;
		};
		SetTextStyle(TabText);
		DrawTabText(as);
		NewLine();
	}
}
