/***************************************************************************
                          stdwheelchart.cpp  -  description
                             -------------------
    begin                : Sat Dec 30 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stdwheelchart.h"
#include "astrorings.h"
#include "astrodefs.h"
#include "astroresources.h"

#define HEIGHT 38

StdWheelChart::StdWheelChart(ChartBaseParams *cbp, ObjsVector* ov) : GraphicChartBase(cbp, ov, cbp->Ag)
{
  Ar = 0;
  Asb = Asb1 = 0;
  Aaf = 0;
  ct = StdWheels;
  cm = GraphicMode;
  DualChart = FALSE;
  DrawChart();
}

StdWheelChart::~StdWheelChart()
{
  if(Ar)   delete Ar;
  if(Asb)  delete Asb;
  if(Asb1) delete Asb1;
  if(Aaf) delete Aaf;
}

void StdWheelChart::DrawChart()
{
  int w, s, size = Ag->GetSize(), h = size * HEIGHT;
  AstroString as;
  First = (*this)[Search(1, Any, TRUE)];
  s = Search(2, Any, TRUE);
  Second = (s != -1) ? (*this)[s] : 0;
  if(Ar)   delete Ar;
  if(Asb)  delete Asb;
  if(Asb1) delete Asb1;
  if(Aaf) delete Aaf;
  //ComputeAspects();
  if(Cbp->Sidebars)
  {
    if(Count() > 1)
    {
      Asb = new AstroSideBar(Ag, 0, size , size * 12, h - size, Cbp, this, Second);
      CHECK_PTR(Asb);
      w = Asb->GetWidth();
      Ar = new AstroRings(Ag, w, size, h, Cbp, this, Afal, First, Second);
      CHECK_PTR(Ar);
      w += Ar->GetWidth();
      Asb1 = new AstroSideBar(Ag, w, size, size * 12, h - size, Cbp, this, First);
      CHECK_PTR(Asb1);
      w += Asb1->GetWidth();
    }
    else
    {
      Ar = new AstroRings(Ag, 0, size, h, Cbp, this, Afal, First, Second);
      CHECK_PTR(Ar);
      w = Ar->GetWidth();
      Asb = new AstroSideBar(Ag, Ar->GetWidth(), size, size * 12, h - size, Cbp, this, First->Show ? First : Second);
      CHECK_PTR(Asb);
      w += Asb->GetWidth();
      Asb1 = 0;
    }
  }
  else
  {
    Ar =  new AstroRings(Ag, 0, size, size * HEIGHT, Cbp, this, Afal, First, Second);
    CHECK_PTR(Ar);
    w = Ar->GetWidth();
    Asb = Asb1 = 0;
  }
  if(Cbp->AspectsFrame)
  {
   Aaf = new AstroAspectsFrame(Ag, 0, h + size, w, 0, Cbp, this, &Afal, First, Second);
   CHECK_PTR(Aaf);
   //SetHeight(size * HEIGHT + Aaf->GetHeight());
  }
  else
  {
   //SetHeight(size * HEIGHT);
   Aaf = 0;
  }
  SetHeight(size * HEIGHT);
  SetWidth(w);
}

void StdWheelChart::ComputeAspects()
{
  AstroAspects aa(Second ? ~(*Second) : ~(*First));
  if(Second)
  {
   if( !(First->GetSubType() == Single && Second->GetSubType() == Single))
    Afal.GetList(aa, *Second, *First, TRUE);
  }
  else
   Afal.GetList(aa, *First, *First, FALSE);
}

bool StdWheelChart::Redisplay()
{
	ComputeAspects();
  if(IfRedrawChart)
  {
    DrawChart();
    IfRedrawChart = FALSE;
  }
  //else
   //ComputeAspects();
  Ar->Repaint();
  if(Asb)  Asb->Repaint();
  if(Asb1) Asb1->Repaint();
  if(Aaf) Aaf->Repaint();
  return TRUE;
}
